#!/usr/bin/perl 

# Copyright 2003 The Board of Trustees of 
# The Leland Stanford Junior University. All Rights Reserved.

# Communication to Jessica Shapiro, jessicas@stanford.edu

use lib "/var/www/cgi-bin/FoldMiner2"; #needed so that Scop.pm and Expectation.pm are found when running in taint mode
use strict vars;   #requires variables to be declared

use CGI;
use Scop qw(%num_SSEs %classifications %numdomains %foldnames $VERSION); #has a hash (%classifications) with scop domains as keys and classifications as values
use Expectation qw(%lambda %mu %lambda_local %mu_local); #has hashes %lambda and %mu containing expectation curve parameters for folds

$ENV{'PATH'} = "/bin:/usr/bin:/usr/local/bin";

my($MIN_CHIME_SIZE) = 200;
my($MAX_CHIME_SIZE) = 500;
my($DEFAULT_CHIME_SIZE) = 300;
my($databasedir) = "/var/www/html/FoldMiner2/targetdb";  #astral files located here
my($id, $chime, $bgdistribution, %foldcounts, %descriptioncounts);
my($filenum,$targetdb,$globalquery,$expectation, $resultsdir, $querywwwlink, $xval, $searchiter,$global); #arguments to this script
my($base,$exponent);
my($chimesize);
my($queryclass,$queryfold,$querysuperfam,$queryfam);
my($databasesize, $pcutoff, $stringent_pcutoff,$outputdir);
my($min_aligned_SSEs) = 3; #minimum number of SSEs that must be aligned, else alignment is discarded (make this a parameter?)
my($change_chime_size);

#some variables
my($line,$i,@fields, @hits, $converged, $globalquerymax);
my($refined_max_score, $num_conserved_SSEs, $old_score);
my(%dp_scores_max, %pvalues_max);
my(%aligned_SSE_counts, %conserved_query_SSEs, %conservation);
my(%SSE_scores);
my(%maxscores);
my(%num_aligned_SSEs,%num_aligned_residues,%RMSD,%description,%finalscores, $default_script);
my($SSEs, $type,$dummy, @query_SSE_starts, @query_SSE_ends, @query_SSE_types, $query_SSEs);
my($start,$end);
my($sendresponse, $test, @exclude, $excludestring, %excluded, $excludescore, $exclude_SSEs,$indexframecode, $indexscript,$indexframecode2,$sortcol);
my($percentID, $scopLevel);
my($q); #CGI object

$q = new CGI;

$id = $q->param('id');
$chime = $q->param('chime');
$bgdistribution = $q->param('bgdist');

#get global parameter
if(defined($q->param('global'))) {
  $global = $q->param('global');
  if($global =~ /^([01])$/) {
    $global = $1;
  }else {
    redirectToErrorHandler($global,"The value you submitted for the alignment scope (i.e. global vs. local) has an unexpected value.","1 (global alignments),0 (local alignments)","alignmentscope",$id);
    exit(-1);
  }
}else {
  $global = 1;
}


#check validity of parameters chime, bgdist, and id
if(defined($bgdistribution)) {
  if($bgdistribution =~ /^(all|[abcd]\.[0-9]+)$/) {
    $bgdistribution = $1;
  }else {
    redirectToErrorHandler($bgdistribution,"You have submitted a background score distribution that has an unexpected value. If you are not submitting this request via the FoldMiner web pages, please make sure that your value for the bgdist variable corresponds to one of the available options.","all,a,a.1","significance",$id);
    exit(-1);
  }
}
if(defined($chime)) {
  if($chime =~ /^([01])$/) {
    $chime = $1;
  }else { 
    redirectToErrorHandler("","An internal variable (chime) has an unexpected value. If you are not submitting this search via the FoldMiner homepage, please make sure that you do not change the values of hidden variables.","","","$id");    
    exit(-1);
  }
}

if(defined($id)) {
  if($id =~ /^([0-9]+)$/) {
    $id = $1;
    $outputdir = "/FoldMiner2/results/$id";
    $resultsdir = "/var/www/html/FoldMiner2/results/$id";
  }else {
    redirectToErrorHandler("","An internal variable (id) has an unexpected value. If you are not submitting this search via the FoldMiner homepage, please make sure that you do not change the values of hidden variables.","","",$id);    
    exit(-1);
  }


  $expectation = $q->param('expectation');
  $xval = $q->param('xval');
  $filenum = $q->param('filenum');

  #check validity of parameters expectation, xval, and filenum
  if($xval =~ /^([10])?(\.[0-9]*)?$/ && $xval >= 0 && $xval <=1) {
    $xval = $1.$2;
    $xval += 0; #make sure it's interpreted as a number
  }else {
    redirectToErrorHandler($xval,"The value you submitted for the conservation profile's contribution to recalculated SSE scores has an unexpected value. The value must be a number between 0 and 1.","0,0.5,.5,1.0","conservation",$id); 
    exit(-1);
  }
  if($expectation =~  /^([0-9]*)(\.[0-9]*)?$/) {
    $expectation = $1.$2;
  }elsif($expectation =~ /^([0-9]+e-?[0-9]+)?$/){
    $expectation = $1;
  }else {
    redirectToErrorHandler($expectation,"The mantissa value you submitted has an unexpected value. The value must be a positive number.","10,0.1,.1","expectation",$id);
    exit(-1);
  }

#need to retrieve filenum from resultsdir
  open(IN,"$resultsdir/filenum.txt") || print STDERR "can't open $resultsdir/filenum.txt: $!\n";
  $filenum = <IN>;
  chomp($filenum);
  if($filenum =~ /^([0-9]*)$/) {
    $filenum = $1;
  }else {
    redirectToErrorHandler($filenum,"The variable 'filenum' has an unexpected value.","","",$id);
  }
  close(IN);

  $targetdb = $q->param('db');
  #check the target database filename and make sure it is legal
  my($temptargetdb) = $targetdb;
  $targetdb = checkTargetDBFilename($targetdb);

  if($targetdb eq "") { #there was a problem with the filename
    my(@tempfields);
    @tempfields = split(/\//,$targetdb);
    redirectToErrorHandler(@tempfields[$#tempfields],"The target database you specified either does not exist or contains incorrect syntax. If you are not submitting this request via the FoldMiner web pages, please make sure that the value of the db variable corresponds to one of the available options.","","searchsubset",$id);
    exit(-1); #need to do error checking in script that calls this
  }
  printWaitPage($filenum,$id,$expectation,$xval,$chime,$targetdb);
  exit(0);
}

$sortcol = $q->param('sortcol'); #set on table page if sorting by some column
$chimesize = $q->param('chimesize');
if(!defined($sortcol)) {
  $sortcol = -1;
  $targetdb = $q->param('db');
  $sendresponse = $q->param('newsearch');
  $base = $q->param('base');
  $exponent = $q->param('exponent');
  $expectation  = $base * (10 **$exponent);
}
$excludestring = $q->param('exclude');
$globalquery = $q->param('query');
$querywwwlink = $q->param('querylink');
$resultsdir = $q->param('resultsdir');
$xval = $q->param('xval');
$searchiter = $q->param('iter'); #how many searches have we already done?
$filenum = $q->param('filenum');
if($filenum =~/^([0-9]+)$/) {
  $filenum = $1;
}else {
  redirectToErrorHandler($filenum,"An internal variable (filenum) has an unexpected value. If you are not submitting this request via the FoldMiner homepage, please make sure that you do not change the values of hidden variables.","","",$id);
  exit(-1);
}

#make sure the query is legal
my($tempqueryname) = $globalquery;
if(!$globalquery eq "query") {
  if(length($globalquery) == 7) { #using scop
    $globalquery = securityCheckProteinName($globalquery,1);
  }else {
    $globalquery = securityCheckProteinName($globalquery,0);
  }
}
if($globalquery eq "") { #there was a problem with $globalquery
#  print STDERR "findmotif.cgi: problem with query name\n";
  redirectToErrorHandler($tempqueryname,"Query PDB or SCOP id is missing or incorrectly formatted","d1f5wa1,1f5w,1f5w-A","query",$id);
  exit(-1);
}

#make sure resultsdir is legal
if($resultsdir =~m|^(/FoldMiner2/results/[0-9]+)$|) {
  $resultsdir = "/var/www/html$1";
  $outputdir = $1;
}else {
  redirectToErrorHandler("","An internal variable (resultsdir=$resultsdir) has an unexpected value. If you are not submitting this request via the FoldMiner homepage, please make sure that you do not change the values of hidden variables.","","",$id);
  exit(-1);
}


my($wwwdir) = $q->param('wwwdir');
if(defined($wwwdir) && $wwwdir ne "") {
  $querywwwlink .= "&resultsdir=$resultsdir&wwwdir=$wwwdir";
}
  
#check validity of expectation parameter
if($expectation =~  /^([0-9]*)(\.[0-9]*)?$/) {
  $expectation = $1.$2;
}elsif($expectation =~ /^([0-9]+e-?[0-9]+)?$/){
    $expectation = $1;
}else {
  redirectToErrorHandler($expectation,"The mantissa value you submitted has an unexpected value. The value must be a positive number.","10,0.1,.1","expectation",$id);
  exit(-1);
}

#check validity of xval parameter
if($xval =~ /^([10])?(\.[0-9]*)?$/ && $xval >=0 && $xval <=1) {
  $xval = $1.$2;
  $xval += 0; #make sure it's interpreted as a number
}else {
  redirectToErrorHandler($xval,"The value you submitted for the conservation profile's contribution to recalculated SSE scores has an unexpected value. The value must be a number between 0 and 1.","0,0.5,.5,1.0","conservation",$id); 
  exit(-1);
}


#check validity of searchiter parameter
if($searchiter =~ /^([0-9]+)$/) {
  $searchiter = $1;
}else {
  redirectToErrorHandler("","An internal variable (searchiter=$searchiter) has an unexpected value. If you are not submitting this search via the FoldMiner homepage, please make sure that you do not change the values of hidden variables.","","","$id");    
  exit(-1);
};

&readSearchFile($resultsdir,$sortcol,$filenum); #read in info on search results

if($sortcol == -1) { #if sortcol==-1, we must analyze the alignment results
  #need to retrieve filenum from resultsdir
  open(IN,"$resultsdir/filenum.txt") || print STDERR "can't open $resultsdir/filenum.txt: $!\n";
  $filenum = <IN>;
  chomp($filenum);
  if($filenum =~ /^([0-9]*)$/) {
    $filenum = $1;
  }else {
    redirectToErrorHandler($filenum,"The variable 'filenum' has an unexpected value.","","",$id);
  }
  close(IN);

  my($classification,$domain,$scopid,$pdbid,$chain);
  $classification = "";
  if(defined($bgdistribution)) { #user selected own distribution
    $queryfold = $bgdistribution;
  }else {
    if(defined($classifications{$globalquery})) {
      $classification = $classifications{$globalquery};
    }elsif($globalquery ne "query") {
      #have pdb id
      if(length($globalquery) == 4) {
	$pdbid = substr($globalquery,0,4);
	$chain = "_";
      }elsif(length($globalquery)==6) {
	$pdbid = substr($globalquery,0,4);
	$chain = substr($globalquery,5,1);      
      }
      $scopid = "d".$pdbid.$chain."_";
      for $domain (keys(%classifications)) {
	if($domain eq $scopid) {
	  $classification = $classifications{$domain};
	}
      }
    }
    #get SCOP classification, if available
    if(defined($classifications{$globalquery})) {
      ($queryclass,$queryfold,$querysuperfam,$queryfam) = split(/\./,$classifications{$globalquery});
      
      #expectation values are hashed in the form of <class>.<fold>
      $queryfold = $queryclass.".".$queryfold;

      # if we do not have a background distribution for this query, 
      # use the overall distribution for all of SCOP
      if(!defined($lambda{$queryfold})) {
	#need to assign background distributions based on class, if possible, 
	# or overall distribution if this is not possible
	if($queryclass eq "a") {
	  $queryfold = "a";
	}elsif($queryclass eq "b") {
	  $queryfold = "b";
	}elsif($queryclass eq "c") {
	  $queryfold = "c";
	}elsif($queryclass eq "d") {
	  $queryfold = "d";
	}
	$queryfold = "all"; #always use overall distribution for now
      }
    }else {
      $queryfold = "all";
    }
  }

  #check the target database filename and make sure it is legal
  my($temptargetdb) = $targetdb;
  $targetdb = checkTargetDBFilename($targetdb);

  if($targetdb eq "") { #there was a problem with the filename
    my(@tempfields);
    @tempfields = split(/\//,$targetdb);
    redirectToErrorHandler(@tempfields[$#tempfields],"The target database you specified either does not exist or contains incorrect syntax. If you are not submitting this request via the FoldMiner web pages, please make sure that the value of the db variable corresponds to one of the available options.","","searchsubset",$id);
    exit(-1); #need to do error checking in script that calls this
  }
  
  #check validity of the base parameter
  if(!defined($base)) {
    $base = 1;
  }elsif($base =~ /^([0-9]+)(\.[0-9]*)?$/) {
    $base = $1.$2;
  }else {$base = $expectation;}

  if($exponent =~ /^(-?[0-9]+)$/) {
    $exponent = $1;
  }else {$exponent = 1;}

  if(!defined($expectation)) { 
    $expectation = $base * (10 ** $exponent); #this should always be true
  }

  $excludescore = 0;
  $exclude_SSEs = 0;

  if(defined($excludestring) && $excludestring ne "") { #SSEs to exclude
    $#exclude = -1;
    @fields = split(",",$excludestring);
    for($i=0;$i<=$#fields;$i++) {
      $exclude_SSEs++;
      $exclude[$#exclude+1] = $fields[$i];
      $excluded{$fields[$i]} = 1;
    }
    if($#exclude != -1) {
      $searchiter++; #since we excluded SSEs, we must create new iteration
    }
  }

  #check validity of the sendresponse parameter
  if($sendresponse =~/^([01])$/) {
    $sendresponse = $1;
  }else {
    redirectToErrorHandler("","An internal variable has an unexpected value. If you are not submitting this search via the FoldMiner homepage, please make sure that you do not change the values of hidden variables.","","","$id");    
    exit(-1);
  }
  
  #calculate the p value needed to ensure a given expectation
  $pcutoff = $expectation/$databasesize;
  $stringent_pcutoff = $pcutoff/10.0; #could make this a parameter
}




my($logfile) = "$resultsdir/logfile";
open(LOG, $logfile) || die "can't open $resultsdir/logfile: $!\n";
while($line = <LOG>) {
  last if($line =~/^Query SSEs:/);
}
chomp($line);

  
($dummy,$SSEs) = split(": ",$line);
@fields = split(/ /,$line);
shift(@fields);
shift(@fields);
$#query_SSE_starts = -1;
$#query_SSE_ends = -1;
$#query_SSE_types = -1;
$query_SSEs = scalar(@fields); #number of query SSEs

#put start and end residues of query SSEs in arrays
for($i=0;$i<=$#fields;$i++) {
  ($query_SSE_types[$#query_SSE_types+1],$query_SSE_starts[$#query_SSE_starts+1],$query_SSE_ends[$#query_SSE_ends+1]) = split(",",$fields[$i]);
}
close(LOG);

if($sortcol == -1) { #need to analyze alignment results
  %dp_scores_max = ();
  $converged = 0;
  $globalquerymax = 40*($query_SSEs-$exclude_SSEs) - 20; #maximum alignment score
  
  
  &run_query($globalquery,$query_SSEs-$exclude_SSEs,$globalquerymax,0, $queryfold); #does initial run based on full dp score
  
  
  @hits = ();
  $old_score = $globalquerymax; #initialization
  while(!$converged) {
    
    ($refined_max_score,$num_conserved_SSEs,@hits) = find_conserved_SSEs($stringent_pcutoff,$pcutoff);

    #check to see if there are no hits
    if($#hits == -1) {
      $converged=0;
      for($i=0;$i<=$#query_SSE_starts;$i++) {
	$conservation{$query_SSE_starts[$i]} = 1;
      }
      last;
    }

    #check for convergence
    if($refined_max_score < $old_score) {
      if($refined_max_score + 2 >= $old_score) {
	$converged = 1;
	last;
      }
    }elsif ($refined_max_score - 2 <= $old_score){
      $converged = 1;
      last;
    }
    
    $old_score = $refined_max_score;
    &run_query($globalquery,$num_conserved_SSEs,$refined_max_score,1, $queryfold);
  }
  
  #print results to files and send response if necessary
  #if this was invoked remotely (vs. by runlock.pl), print content type
  if($sendresponse)  {
    print "Content-type: text/html\n\n"; #part of response header; make sure this gets printed first!
  }

  printProfile($filenum,$resultsdir, $searchiter, $xval);
  printSearchTable($filenum,$searchiter, $xval,$expectation,-1,$DEFAULT_CHIME_SIZE);
  printIterativeSearchOptionsPage($filenum,$searchiter,$xval,$expectation,$queryfold);
  ($indexframecode,$indexscript,$indexframecode2)=printIndex($filenum,$searchiter,$xval,$expectation);
  printMotifPage($filenum,$resultsdir,$globalquery, $searchiter, $xval, $percentID, $scopLevel, $id,$indexframecode,$indexscript,$indexframecode2,$chime, $expectation, $queryfold,$DEFAULT_CHIME_SIZE);

  open(OUT, ">$resultsdir/filenum.txt") || die;
  printf OUT "%i",$filenum+1;
  close(OUT);

  #this needs to be done last so that table, motif, and profile are already in place
  printResultsPage($filenum,$searchiter, $xval,$expectation,$indexframecode,$indexscript,$indexframecode2,0); 
}else {
  $change_chime_size = $q->param('changechimesize');

  if(!defined($change_chime_size) || $change_chime_size==0) {
    printSearchTable($filenum,$searchiter, $xval, $expectation,$sortcol,$chimesize);
  }else { 
    if($change_chime_size =~ /^(1)$/) {
      $change_chime_size = $1;
    }else {
      redirectToErrorHandler($change_chime_size,"The variable 'change_chime_size' has an unexpected value.","","",$id);
      exit(-1);
    }    

    #check validity of chimesize parameter
    if($chimesize =~ /^([1-9]+[0-9]+)$/) {
      $chimesize=$1;
    }else {
      redirectToErrorHandler($chimesize,"An internal variable (chimesize=$chimesize) has an unexpected value.","","","$id");    
      exit(-1);
    }
    $filenum = $q->param('filenum');
    if($filenum =~ /^([0-9]*)$/) {
      $filenum = $1;
    }else {
      redirectToErrorHandler($filenum,"The variable 'filenum' has an unexpected value.","","",$id);
    }
    #need to read in profile (SSE conservation values) -- would be nice to store this as a cookie?
    open(PROFILE, "$resultsdir/profile$filenum");
    my($cons);
    while($line = <PROFILE>) {
      chomp($line);
      ($start,$end,$cons,$dummy) = split(/\s+/,$line);
      $conservation{$start} = $cons;
    }
    close(PROFILE);
    
    #need to read in index frame code
    open(INDEX, "$resultsdir/results\.html");
    while($line = <INDEX>) {
      last if($line =~/function writeControlPanel/);
    }
    <INDEX>;
    $indexframecode = <INDEX>;
    $indexscript=<INDEX>;
    $indexframecode2=<INDEX>;
    $indexframecode =~ s/controlwin.document.writeln\(//;
    $indexscript =~ s/controlwin.document.writeln\(//;
    $indexframecode2 =~ s/controlwin.document.writeln\(//;
    $indexframecode =~ s/\);$//;
    $indexscript =~ s/\);$//;
    $indexframecode2 =~ s/\);$//;
    close(INDEX);

    printMotifPage($filenum,$resultsdir,$globalquery, $searchiter, $xval, $percentID, $scopLevel, $id,$indexframecode,$indexscript,$indexframecode2,$chime, $expectation, $queryfold,$chimesize);
    $sendresponse = 1;
    print "Content-type: text/html\n\n"; #part of response header; make sure this gets printed first!
    printResultsPage($filenum,$searchiter, $xval,$expectation,$indexframecode,$indexscript,$indexframecode2,1); #this needs to be done last so that table, motif, and profile are already in place
  }
}

sub find_conserved_SSEs {
  my($stringent_pcutoff, $pcutoff)= @_;
  my($i,$j);
  my(@orderbyscore1);
  my($l);
  my($last_index);
  my($type);
  my($line,@fields);
  my($start,$end,$start2,$end2,$temp,$num_results,$num_conserved,$conserved_total);
  my($total_contribution,$target,$file, $scorefield, @SSEs, %scores);
  my($SSE);
  my($found_alignment_start);
  my(%total_contributions,%aligned);


  #sort hits by score
  @orderbyscore1 = sort(sort_by_score keys(%dp_scores_max));
  if($#orderbyscore1==-1) { #no statistically significant results
    return(0,0,@orderbyscore1);
  }

  #find all hits that meet the stringent statistical significance cutoff
  $last_index = $#orderbyscore1;
  for($i=0;$i<=$#orderbyscore1;$i++) {
    if($pvalues_max{$orderbyscore1[$i]} > $stringent_pcutoff)  {
      $last_index = $i-1;
      last;
    }
  }
  
  #get lambda such that lowest scoring target contributes 0.01
  #to new maximum score
  if($last_index != $#orderbyscore1) { 
    $l = inverse_exponential_function(($#orderbyscore1-$last_index), 0.01);
  }elsif($#orderbyscore1 != 0) {
    $l = inverse_exponential_function($#orderbyscore1,0.01); #if no hits made stringent cutoff
  }


  %aligned_SSE_counts = ();
  %conserved_query_SSEs = ();
  %total_contributions = ();
  for($i=0;$i<=$#query_SSE_starts;$i++) {
    $aligned_SSE_counts{$query_SSE_starts[$i]} = 0;
    $conserved_query_SSEs{$query_SSE_starts[$i]} = 0; #not conserved by default
    $total_contributions{$query_SSE_starts[$i]} = 0;
  }
  
  
  #find aligned SSEs in the significant results
  $total_contribution = 0;
  for($i=0;$i<=$#orderbyscore1;$i++) {
    $target = $orderbyscore1[$i];
    if($i>$last_index) {
      $total_contribution += exp(-$l*($i-$last_index));
    }else {
      $total_contribution += 1;
    }
    #need to extract keys from %SSE_scores{$target} 
    %scores = %{$SSE_scores{$target}}; #aligned SSEs for this target are keys, scores are values
    %aligned = ();
    for($j=0;$j<=$#query_SSE_starts;$j++) {
      $aligned{$query_SSE_starts[$j]} = 0;
    }   
    @SSEs = keys(%scores);
    foreach $SSE (@SSEs) {
      next if(defined($excluded{$SSE}));
      if($i>$last_index) {
	$temp = exp(-$l * ($i-$last_index));
      }else {
	$temp = 1;
      }
      $aligned_SSE_counts{$SSE} += $scores{$SSE}*$temp;
    }
    $found_alignment_start = 0;
    for($j=0;$j<=$#query_SSE_starts;$j++) {
      $SSE = $query_SSE_starts[$j];
      next if(defined($excluded{$SSE}));
      if($j==0) {
	$total_contributions{$query_SSE_starts[$j]} += 20*$temp;
      
	if($aligned{$query_SSE_starts[$j]}) {
	  $found_alignment_start = 1;
	}
      
      }else {
	if(!$found_alignment_start && $aligned{$query_SSE_starts[$j]}) {
	  $total_contributions{$query_SSE_starts[$j]} += 20*$temp;
	  $found_alignment_start = 1;
	}else {
	  $total_contributions{$query_SSE_starts[$j]} += 40*$temp;
	}
      }
    }   
  } #ends loop over targets

  #select conserved SSEs
  $num_results = $#orderbyscore1+1;
  $num_conserved = 0;
  $refined_max_score = 0;
  
  $conserved_total = 0;
  for($i=0;$i<$query_SSEs;$i++) {
	next if($excluded{$query_SSE_starts[$i]}==1);
    #make score weight proportional to conservation
    if($i != 0) {
      if($total_contributions{$query_SSE_starts[$i]} != 0) {      
	$conserved_query_SSEs{$query_SSE_starts[$i]} = (1-$xval) + $xval*($aligned_SSE_counts{$query_SSE_starts[$i]}/$total_contributions{$query_SSE_starts[$i]});
	$conservation{$query_SSE_starts[$i]} = ($aligned_SSE_counts{$query_SSE_starts[$i]}/$total_contributions{$query_SSE_starts[$i]});
      }else {
	$conserved_query_SSEs{$query_SSE_starts[$i]} = 1-$xval;
	$conservation{$query_SSE_starts[$i]} = 0;
      }
    }else {
      if($total_contributions{$query_SSE_starts[$i]} != 0) {
	$conserved_query_SSEs{$query_SSE_starts[$i]} = (1-$xval) + $xval*($aligned_SSE_counts{$query_SSE_starts[$i]}/$total_contributions{$query_SSE_starts[$i]});
	$conservation{$query_SSE_starts[$i]} = ($aligned_SSE_counts{$query_SSE_starts[$i]}/$total_contributions{$query_SSE_starts[$i]});
      }else {
	$conserved_query_SSEs{$query_SSE_starts[$i]} = 1-$xval;
	$conservation{$query_SSE_starts[$i]} = 0;
      }
    }

    $num_conserved += $conserved_query_SSEs{$query_SSE_starts[$i]};
    $refined_max_score += 40*$conserved_query_SSEs{$query_SSE_starts[$i]};
    if($i==0) { #first SSE is worth only half of max SSE score
      $refined_max_score -= 20*$conserved_query_SSEs{$query_SSE_starts[0]};
    }
  }

  return(($refined_max_score,$num_conserved,@orderbyscore1));
}


#examines all alignments and finds statistically significant hits
sub run_query {
    my($query,$max_conserved_SSEs,$querymax,$use_conserved, $queryfold) = @_;
    my($line,$file,@fields);
    my($analysis_file);
    my($query_target_size_ratio);
    my($targetmax,$target,@targets,$score,%scores,@SSEs, $SSE,$pval,$newtargetmax);

    %dp_scores_max = ();

    return if($querymax==0);
    return if($query_SSEs < 1);
    @targets = keys(%SSE_scores); #aligned targets
    foreach $target (@targets) {
      next if($classifications{$target} !~ /^[abcd]/); #use only four main classes
      %scores = %{$SSE_scores{$target}}; #aligned SSEs for this target are keys, scores are values

      @SSEs = keys(%scores);
      @SSEs = sort {$a <=> $b} @SSEs;
      #find the score for this target
      $score = 0;
      if(!defined($maxscores{$target})) {
	$maxscores{$target} = 40*$num_SSEs{$target}-20;
      }
      if(!$use_conserved) {
	$newtargetmax = $maxscores{$target};
	for($i=0;$i<=$#SSEs;$i++) {
	    $SSE = $SSEs[$i];
	    next if($excluded{$SSE} == 1);
	  $score += $scores{$SSE};
	}
      }else {
	$targetmax = $maxscores{$target};
	$newtargetmax = $targetmax;
	for($i=0;$i<=$#SSEs;$i++) {
	  $SSE = $SSEs[$i];
	  next if($excluded{$SSE} == 1);
	  #each SSE score is weighted by its conservation, which is given by $conserved_query_SSEs{$SSE}
	  $score += $scores{$SSE}*$conserved_query_SSEs{$SSE};
	  if($i==0) {
	    $newtargetmax -= (20-20*$conserved_query_SSEs{$SSE});
	  }else {
	    $newtargetmax -= (40-40*$conserved_query_SSEs{$SSE});
	  }
	}
      }
      next if($targetmax ==0 && $querymax == 0);
      next if(scalar(@SSEs) < $min_aligned_SSEs); #ignore target if too few SSEs are aligned

      # Normalization is based on number of SSEs in the larger of the query and target
      # structures in order to favor alignments that are global with respect to both
      # the query and target. If the query is larger, the score is normalized to the current
      # query vs. query alignment score. If the target is larger, we multiply the original
      # target vs. target alignment score by the ratio of the current maximum query vs. query
      # alignment score to the original query vs. query alignment score. 
      if($num_SSEs{$target} > $query_SSEs) {
	if($global) {
	  $score = $score/($maxscores{$target}*$querymax/$globalquerymax);
	}else {
	  $score = $score/$querymax;
	}
      }else {
	if($global) {
	  $score = $score/$querymax;
	}else {
	    if($newtargetmax != 0) {
		$score = $score/$newtargetmax;
	    }else {
		$score = 0;
	    }
	}
      }

      $finalscores{$target} = $score;
      if($score > 0) {
	if($global) {
	  $pval = &expectation_function($lambda{$queryfold},$mu{$queryfold},$score);
	}else {
	  $pval = &expectation_function($lambda_local{$queryfold},$mu_local{$queryfold},$score);
	}
	$pvalues_max{$target} = $pval;
	if($pval <= $pcutoff) {
	  $dp_scores_max{$target} = $score;
	}
      }else {
	$pvalues_max{$target} = 1;
      }
    }

}

sub printProfile {
  my($filenum,$resultsdir, $searchiter, $xval) = @_;
  my($file);
  my($i);
  
  #give each conservation profile a unique file name 
  $file = ">$resultsdir/profile$filenum";

  open(PROFILE, $file) || die "can't open $file: $!\n";

  for($i=0;$i<=$#query_SSE_starts;$i++) {
    print PROFILE "$query_SSE_starts[$i] $query_SSE_ends[$i] $conservation{$query_SSE_starts[$i]} $conserved_query_SSEs{$query_SSE_starts[$i]}\n"
  }
  close(PROFILE);
}

#prints the frame in which the query motif is displayed using chime
sub printMotifPage {
  my($filenum,$resultsdir,$query, $searchiter,$xval,$percentID, $scopLevel, $id,$indexframecode,$scriptcode,$indexframecode2,$chime, $expectation,$queryfold,$chimesize) = @_;
  my($newfilenum,$line, @starts,@ends,@conservations, $table_file);
  my($i,@color,@color2);
  my($colorstring, $middle, $label);
  my($scriptstring, $colorstring2, $low, $high,$intervalsize, $ratio);
  my($databasestring,$expectationstring);
  my($spanopen,$spanclose,$num,$newiter);
  my($shortqueryname);


  if($globalquery =~ /^$resultsdir\/(queryfile)\.pdb$/) {
    $shortqueryname = $1;
  }else {
    if($globalquery =~/^($globalquery)$/) {
      $shortqueryname = $1;
    }
  }
  

  $newfilenum = $filenum+1;

  if($percentID != -1) {
    $databasestring = "<= $percentID\% sequence identity";
  }elsif($scopLevel != -1) {
    $databasestring = "One representative from each <a href=\"http://scop.berkeley.edu/\" target=_blank>SCOP</a> ";
    if($scopLevel eq "cl") {
      $databasestring .= "class.";
    }elsif($scopLevel eq "cf") {
      $databasestring .= "fold.";
    }elsif($scopLevel eq "sf") {
      $databasestring .= "superfamily.";
    }elsif($scopLevel eq "fa") {
      $databasestring .= "family";
    }elsif($scopLevel eq "dm") {
      $databasestring .= "protein.";
    }else {
      $databasestring = "Can't identify your target database!";
    }
    
  }else {
    $databasestring = "Can't identify your target database!";
  }

  if($queryfold eq "all") {
    $expectationstring = "<li><b>The query is either not in SCOP or belongs to a SCOP fold for which a background alignment score distribution does not exist.</b><ul><li> A background score distribution encompassing all of SCOP was therefore used to analyze your results.<li> If you would like to use a more specific distribution, you may <a href=\"$outputdir/iterate$filenum\.html#reanalyze\" target=\"iterate\">reanalyze</a> these results.</ul>\n";
  }elsif($queryfold =~ /^[abcd]$/) {
    
    $expectationstring = "<li><b>The query is either not in SCOP or belongs to a SCOP fold for which a background alignment score distribution does not exist.</b><ul><li> A background score distribution encompassing";
    if($queryfold eq "a") {
      $expectationstring .= " the <a href=\"http://scop.berkeley.edu/search.cgi?sccs=a\" target='scop'>mainly alpha SCOP</a> class";
    }elsif($queryfold eq "b") {
      $expectationstring .= " the <a href=\"http://scop.berkeley.edu/search.cgi?sccs=b\" target='scop'>mainly beta</a> SCOP class";
    }elsif($queryfold eq "c") {
      $expectationstring .= " the <a href=\"http://scop.berkeley.edu/search.cgi?sccs=c\" target='scop'>alpha and beta proteins (a/b)</a> SCOP class";
    }elsif($queryfold eq "d") {
      $expectationstring .= " the <a href=\"http://scop.berkeley.edu/search.cgi?sccs=d\" target='scop'>alpha and beta proteins (a+b)</a> SCOP class";
    }
    $expectationstring .= " was therefore used to analyze your results.<li> If you would like to use a more specific distribution, you may <a href=\"$outputdir/iterate$filenum\.html#reanalyze\" target=\"iterate\">reanalyze</a> these results.</ul>\n";
  }else {
    $expectationstring = "The query structure is a member of SCOP fold $queryfold (<a href=\"http://scop.berkeley.edu/search.cgi?sccs=$queryfold\" target='scop'>$foldnames{$queryfold}</a>). By default, a background score distribution specific to this fold will be used to analyze your results. You may <a href=\"javascript:scrollframe('reanalyze')\">reanalyze your results using a different distribution</a>."
  }

  $table_file = "table$filenum\.html";
  @color=getColor(1);
  @color2=getColor(0.3);
  $#starts = $#ends = $#conservations = -1;

  #produce chime script to color query SSEs by conservation
  $colorstring="color gray\n";
  $high = 0; $low = 1;
  for($i=0;$i<=$#query_SSE_starts;$i++) {
    next if($excluded{$query_SSE_starts[$i]});
    $colorstring .= "select ($query_SSE_starts[$i]-$query_SSE_ends[$i] and not hetero)\n";
    @color = getColor($conservation{$query_SSE_starts[$i]});
    if($conservation{$query_SSE_starts[$i]} < $low) {
      $low = $conservation{$query_SSE_starts[$i]};
    }
    if($conservation{$query_SSE_starts[$i]} > $high) {
      $high = $conservation{$query_SSE_starts[$i]};
    }
    $colorstring .="color [$color[0],$color[1],$color[2]]\n";
  }

  $intervalsize = ($high-$low)/20.0;
  $ratio = (100/20)**(1/19);
  $colorstring2="color monochrome\n";

 
  for($i=0;$i<=$#query_SSE_starts;$i++) {
    next if($excluded{$query_SSE_starts[$i]});

    $colorstring2 .= "select ($query_SSE_starts[$i]-$query_SSE_ends[$i] and not hetero)\n";
    if($intervalsize != 0) {
      @color = getColorByConservation(353,95,int(($conservation{$query_SSE_starts[$i]}-$low)/$intervalsize),$ratio);
      $colorstring2 .="color [$color[0],$color[1],$color[2]]\n";
    }else {
      @color = getColorByConservation(353,95,1,$ratio);
      $colorstring2 .="color [$color[0],$color[1],$color[2]]\n";
    }
  }

  #print out rasmol script for download
  open(RASMOL_SCRIPT, ">$resultsdir/$shortqueryname-SSE-conservation-$xval-$expectation.spt");
  print RASMOL_SCRIPT "select all\nwireframe off\ncartoons\nbackground white\n";
  print RASMOL_SCRIPT $colorstring2;
  print RASMOL_SCRIPT "select all\n";
  close(RASMOL_SCRIPT);  


  open(motif, ">$resultsdir/motif$filenum\.html");

  #this script labels query SSEs by conservation
  $scriptstring = "";
  for($i=0;$i<=$#query_SSE_starts;$i++) {
    $middle = int(($query_SSE_ends[$i]-$query_SSE_starts[$i])/2)+$query_SSE_starts[$i];
    $label = int($conservation{$query_SSE_starts[$i]}*100);
    $label/=100;
    $scriptstring .= "set fontsize 7\nselect ($middle and *.CA and not hetero)\nlabel $label;\ncolor label green\n";
  }
#need to do table within table for script options
  print motif <<END;
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html>
<head>
<meta http-equiv:"no-cache"> <!--always load page from server in case it has changed-->
<title>Core Fold/Motif</title>
<link rel=stylesheet href="/FoldMiner2/lock.css" type="text/css">
<script language="Javascript" type="text/javascript">
  var tablewin;
  var controlwin;
  var windowsopened = 0;
  var dir = "$outputdir";
  var chime = $chime;
  window.name='motif$id'

  function scrollframe(anchor) {
    parent.iterate.location.hash=anchor;
  }

  function launchHelpWindow(anchor) {
   var URL = "/FoldMiner2/help.html#"+anchor;
   var mywin=window.open('','help',"directories=no,location=no,menubar=no,scrollbars=yes,toolbar=no,resizable=yes,status=yes,width=350,height=300");
   mywin.location.replace(URL);
    mywin.focus();
  }

  function openWindows() {
      openTable();
      openControlPanel(); 
      window.focus();
      controlwin.focus(); //want this in front and motif window right behind it
      document.myform.openedwindows.value=1;
  }

  //openTable: Opens a table displaying search results. Note that netscape 4.7 (and possibly other versions)
  //do not seem to correctly implement window.open if the page to be displayed is not in the browser\'s
  //cache. Therefore, we open a blank page and replace its URL with the table\'s
  function openTable() {
    tablewin=window.open("", 'table$id');                //open blank page
    tablewin.location.replace('$outputdir/$table_file'); //load table into page 
//    tablewin.opener.focus();  //make sure control panel is in front
  }


  function openControlPanel() {

    if(controlwin == null) {
       controlwin=window.open("", 'control',"status=yes,resizable=yes,scrollbars=yes,location=no,toolbar=no,menubar=no,directories=no,height=300,width=300");
       controlwin.location.replace("$outputdir/index.html");
    }else {
      //controlwin.location.reload();
    }

  }

  function bringTableToFront() {
    top.bringTableToFront();
  }

  function bringControlToFront() {
     top.bringControlToFront();
  }
  

   
   function increaseChimeSize() {
      var chimesize = parseInt(document.chimeform.chimesize.value);
      chimesize += 50;
      document.chimeform.chimesize.value = chimesize.toString();
      document.chimeform.submit();
   }
   
   function decreaseChimeSize() {
      var chimesize = parseInt(document.chimeform.chimesize.value);
      chimesize -= 50;
      document.chimeform.chimesize.value = chimesize.toString();
      document.chimeform.submit();
   }

</script>
</head>

<body bgcolor="#FFFFFF" link="#26256b" alink="#877bab" vlink="#877bab">
<h2 class="header">Query\'s Core Fold/Motif</h2>
<table border=0 cellpadding=2>
<tr><td><a href=/FoldMiner2/ class="smallfont" target=_top>Return to FoldMiner home page</a></td></tr>
<tr><td><a href="#options" class="smallfont">Jump to: What can I do from here?</a></td></tr>
<!--<a href="javascript:bringControlToFront()" class="smallfont">(Bring control panel to front)</a><br>-->
<table border=0 align=center> <!--outer table containing script table and chime display-->

END
#if($chime) {
 print motif <<END;
<tr>
    <td valign=top><embed src=\"$outputdir/$shortqueryname.pdb\"
         bgcolor=white
         frank="no"
         height=$chimesize
         width=$chimesize
         display3D="cartoons"
         name=\"$globalquery\"
         script="$colorstring2; select all">
    </td>
</tr>
<tr><td><a href="#rasmol_scripts"><i><font size=-1>Download rasmol scripts</font></i></a></td></tr>
END
#}else {
# print motif <<END;
#<tr>
#    <td valign=center><p class="strong"><a href="http://www.mdl.com/" target=_blank>Chime</a> does not appear to be installed on your browser. Normally, this page contains a Chime plugin that displays the query structure\'s core fold or motif as identified in the structural similarity search.</p>
#    <a href="#rasmol_scripts"><i><font size=-1>Download Rasmol scripts</font></i></a>
#</td>
#</tr>
#END
#}
  if(!$chime) {
    print motif <<END;
<tr><td><font size=-1><a href="http://www.mdl.com/" target=_blank>Chime</a> is required to view this page. If you do not see a protein structure above, Chime is either not installed or is not functioning properly.</font></td></tr>
END
  }


print motif <<END;

<tr  align=left><td colspan=2><span class="small"><span class="bright">Bright</span> and <span class="dark">dark</span> colors correspond to strongly and weakly conserved secondary structure elements, respectively.</span></td></tr>
<tr>
    <td valign=top><table border=0> <!-- chime options table -->
<!--          <caption align=bottom class="small"><span style="color:rgb(255,13,41)">Bright</span> and <span style="color:rgb(48,2,8)">dark</span> colors correspond to strongly and weakly conserved secondary structure elements, respectively.</caption>-->
         <tr bgcolor="#bbbbbb"><td colspan=2><span class=\"chimeOptionHeader\">Display Options</span></td></tr>
         <tr bgcolor="#f6f6f6">
             <td><a href="javascript:launchHelpWindow('renormalize')" onMouseOver="status='Click for help';return true" onMouseOut="status='';return true" class=\"smallhelplink\">Renormalize color scale to use entire spectrum</a></td>
             <td><embed type=\"application/x-spt\"
                    width=12
                    height=12
                    button=toggle
                    buttonstate=pushed
                    target=\"$globalquery\"
                    script=\"$colorstring2\nselect all\"
                    altscript=\"$colorstring\nselect all\">
             </td>
         </tr>
         <tr bgcolor="#f6f6f6">
             <td><a href="javascript:launchHelpWindow('showconservationvalues')" onMouseOver="status='Click for help';return true" onMouseOut="status='';return true" class=\"smallhelplink\">Show conservation values</a></td>
             <td><embed type=\"application/x-spt\"
                    width=12
                    height=12
                    button=toggle
                    target=\"$globalquery\"
                    script=\"$scriptstring\"
                    altscript=\"$colorstring2\nselect all\nlabel off\">
             </td>
        </tr>
        <tr bgcolor="#f6f6f6">
             <td><span class=\"smallfont\">White background</span></td>
             <td><embed type=\"application/x-spt\"
                    width=12
                    height=12
                    button=push
                    target=\"$globalquery\"
                    script=\"background white\">
             </td>
        </tr>
        <tr bgcolor="#f6f6f6">
             <td><span class=\"smallfont\">Black background</span></td>
             <td><embed type=\"application/x-spt\"
                    width=12
                    height=12
                    button=push
                    target=\"$globalquery\"
                    script=\"background black\">
             </td>
        </tr>
        <form name=chimeform action="/cgi-bin/FoldMiner2/findmotif.cgi" target= _top>
            <input type=hidden name=chimesize value=$chimesize>
            <input type=hidden name=sortcol value=2>
            <input type=hidden name=xval value=$xval>
            <input type=hidden name=iter value=$searchiter>
            <input type=hidden name=resultsdir value=$outputdir>
            <input type=hidden name=query value=$globalquery>
            <input type=hidden name=querylink value="$querywwwlink">
            <input type=hidden name=chime value=$chime>
            <input type=hidden name=filenum value=$filenum>
            <input type=hidden name=changechimesize value=1>
END

if($chimesize < $MAX_CHIME_SIZE) {
print motif <<END;
         <tr><td><input type=button onClick="javascript:increaseChimeSize()" value="Increase chime display size"></td></tr>
END
}
if($chimesize > $MIN_CHIME_SIZE) {

print motif <<END;
         <tr><td><input type=button onClick="javascript:decreaseChimeSize()" value="Decrease chime display size"></td></tr>
END
}
print motif <<END;
         </form>
    <tr><td><a name="rasmol_scripts"></a><i><a href="$outputdir/$shortqueryname-SSE-conservation-$xval-$expectation.spt">Click to download query SSE motif rasmol script</i></a></td></tr>
    </table>
    </td> <!--end chime options table-->
</tr>

<!-- current search options are available in other frame 
<td valign=top>
<table border=0 cellspacing=2>
<tr bgcolor="#bbbbbb"><th align=center colspan=2>Current Search Options</th></tr>
<tr bgcolor="#eeeeee">
  <td align=left>Target Database: </td><td align=right><span class="searchInfo">$databasestring</span></td>
</tr>
<tr>
  <td align=left><a href="javascript:launchHelpWindow('expectation')" onMouseOver="status='Click for help';return true" onMouseOut="status='';return true" class="helplink">Expectation:</a> </td><td align=right><span class="searchInfo">$expectation</span></td>
<tr bgcolor="#eeeeee">
  <td align=left><a href="javascript:launchHelpWindow('conservation')" onMouseOver="status='Click for help';return true" onMouseOut="status='';return true" class="helplink">Conservation Profile\'s Contribution to Recalculated SSE Scores: </td><td align=right><span class="searchInfo">$xval</span></td>
</tr>
<tr>
  <td align=left>Search iteration: </td><td align=right><span class="searchInfo">$searchiter</span></td>
</tr>
END
if($queryfold eq "all") {

}
print motif<<END;
</table>
</td>
-->

<tr>
<td colspan=2> <!--analysis options-->
 <ul>
<a name="options"><li>Your search results have been displayed in a separate browser window <a href="javascript:bringTableToFront()">(bring to front)</a></span>
<li>$expectationstring
<li>Additional data analysis options:
    <ul> 
      <li><a href="javascript:scrollframe('reanalyze')">Analyze current alignment results using different search options</a>
      <li><a href="javascript:scrollframe('iterate')">Iterative motif discovery</a>
    </ul>
<li>Current Search Parameters:
   <ul>
      <li><i><a href="javascript:launchHelpWindow('expectation')" onMouseOver="status='Click for help';return true" onMouseOut="status='';return true" class="helplink">Expectation:</a> $expectation</i>
      <li><i><a href="javascript:launchHelpWindow('conservation')" onMouseOver="status='Click for help';return true" onMouseOut="status='';return true" class="helplink">Conservation profile\'s contribution to recalculated SSE scores:</a> $xval</i>
   </ul>
</ul>
</td>
</tr>

</table>
<br clear="all">
<!--
<hr>
<ul>
<li>Your search results have been displayed in a separate browser window <a href="javascript:bringTableToFront()">(bring to front)</a></span>
$expectationstring
<li>Additional data analysis options:
    <ul>
    <li><a href="javascript:scrollframe('reanalyze')">Analyze current alignment results using different search options</a>
    <li><a href="javascript:scrollframe('iterate')">Iterative motif discovery</a>
    </ul>
</ul>
-->
END

print motif "</body></html>\n";

close(motif);
return($colorstring);
}


sub printIterativeSearchOptionsPage {
  my($filenum,$searchiter, $xval,$expectation, $queryfold) = @_;
  my($newfilenum,$num, $newiter, $spanopen, $spanclose);
  my($diststring, @folds);
  my($distselect, $name);


  $num = $#query_SSE_starts+1; #number of SSEs
  $newiter = $searchiter+1;

  $newfilenum = $filenum+1;
  if($queryfold eq "all") {
    $diststring = "<i>The query structure is either not in SCOP or is a member of a SCOP fold for which a background score distribution does not exist. By default, a background distribution encompassing all of SCOP will be used to analyze your results. You may specifiy a more specific distribution here</i>.";
  }elsif($queryfold =~ /^[abcd]$/) {
    $diststring = "<i>The query structure is either not in SCOP or is a member of a SCOP fold for which a background score distribution does not exist. By default, a background distribution encompassing";
    if($queryfold eq "a") {
      $diststring .= " the <a href=\"http://scop.berkeley.edu/search.cgi?sccs=a\" target='scop'>mainly alpha SCOP</a> class";
    }elsif($queryfold eq "b") {
      $diststring .= " the <a href=\"http://scop.berkeley.edu/search.cgi?sccs=b\" target='scop'>mainly beta</a> SCOP class";
    }elsif($queryfold eq "c") {
      $diststring .= " the <a href=\"http://scop.berkeley.edu/search.cgi?sccs=c\" target='scop'>alpha and beta proteins (a/b)</a> SCOP class";
    }elsif($queryfold eq "d") {
      $diststring .= " the <a href=\"http://scop.berkeley.edu/search.cgi?sccs=d\" target='scop'>alpha and beta proteins (a+b)</a> SCOP class";
    }
    $diststring .= " will be used to analyze your results. You may specifiy a more specific distribution here</i>.";
  }else {
    $diststring = "The query structure is a member of SCOP fold $queryfold (<a href=\"http://scop.berkeley.edu/search.cgi?sccs=$queryfold\" target='scop'>$foldnames{$queryfold}</a>). By default, a background score distribution specific to this fold will be used to analyze your results. You may specify a different distribution here.";
  }

  #sort folds for which we have background distributions
  @folds = keys(%lambda); #get all folds, classes, etc for which we have score distributions
  @folds = sort(sort_folds @folds);

  $distselect = "<select name=bgdist>\n";
  for($i=0;$i<=$#folds;$i++) {
    if($folds[$i] eq "all") {
      $name = "All of SCOP";
    }elsif($folds[$i] =~ /^[abcd]$/) {
      if($folds[$i] eq "a") {
	$name = "Mainly alpha proteins";
      }elsif($folds[$i] eq "b") {
	$name = "Mainly beta proteins";
      }elsif($folds[$i] eq "c") {
	$name = "Alpha and beta proteins (a/b)";
      }elsif($folds[$i] eq "d") {
	$name = "Alpha and beta proteins (a+b)";
      }
    }else {
      $name = "$foldnames{$folds[$i]} ($folds[$i])"; #use fold name instead of number
    }
    if($folds[$i] eq $queryfold) {
      $distselect.=" <option value=$folds[$i] selected>$name\n";      
    }else {
      $distselect.=" <option value=$folds[$i]>$name\n";
    }
  }
  $distselect.="</select>\n";

  open(iter,  ">$resultsdir/iterate$filenum\.html");

print iter<<END;
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html>
<head>
<title>FoldMiner Iterative Search Options</title>
<link rel=stylesheet href="/FoldMiner2/lock.css" type="text/css">
<script language="Javascript" type="text/javascript">

  function bringControlToFront() {
     top.bringControlToFront();
  }

  function bringTableToFront() {
    top.bringTableToFront();
  }
  
  function openWindows() {
    top.openWindows();
  }

  function setExcludeString() {
    var str = "";
    for(var i=0;i<document.myform.elements.length;i++) {
      if(document.myform.elements[i].type == "checkbox") {
	if(document.myform.elements[i].checked) {
	  str += document.myform.elements[i].name;
	  str += ",";
	}
      }
    }
    str = str.substring(0,str.length-1); //remove last comma
    document.myform.exclude.value=str;  
  }
 
  //makes sure an SSE previously excluded from search is not now included, though this is perhaps
  //not necessary
  function checkboxError(box) {
   if(!this.checked) {
      alert("This SSE was excluded in a previous iteration. It cannot be included in future iterations");
      this.checked=true;
   }
 }

  function launchHelpWindow(anchor) {
   var URL = "/FoldMiner2/help.html#"+anchor;
   var mywin=window.open('','help',"directories=no,location=no,menubar=no,scrollbars=yes,toolbar=no,resizable=yes,status=yes,width=350,height=300");
   mywin.location.replace(URL);
    mywin.focus();
  }
</script>
</head>
<body bgcolor="#FFFFFF" link="#26256b" alink="#877bab" vlink="#877bab" onLoad="javascript:setExcludeString();return true">
<i><font size=-1><a href="javascript:openWindows()" class="smallfont">This site requires popup windows to function properly. Click here if your browser blocks them, or disable the popup blocker and reload.</a></font></i><br>
<table border=0 cellpadding=2>
<tr>
<td>
<a href="javascript:bringControlToFront()" class="smallfont">Bring control panel to front</a>
</td>
<td>
<a href="javascript:bringTableToFront()" class="smallfont">Bring search results table to front</a>
</td>
</tr>
</table>
<!--<hr> this is unnecessary if page is split horizontally-->
<A name="iterate"><p>FoldMiner identified the motif displayed in the left frame by determining which secondary structure elements frequently align well to homologous structures. It may be possible to identify additional structural neighbors or additional motifs by excluding poorly or well conserved secondary structural elements, respectively.

<p>To attempt to identify another motif, please select from the list below seconday structure elements that should be <i>excluded</i> from the motif discovery algorithm.

<form name="myform" method=post action="/cgi-bin/FoldMiner2/findmotif.cgi" target=_top>
<input type=hidden name=newsearch value=1>
<input type=hidden name=iter value=$searchiter>
<input type=hidden name=db value=$targetdb>
<input type=hidden name=query value=$globalquery>
<input type=hidden name=resultsdir value=$outputdir>
<input type=hidden name=querylink value=$querywwwlink>
<input type=hidden name=exclude value=''>
<input type=hidden name=chime value=$chime>
<input type=hidden name=openedwindows value=0>
<input type=hidden name=filenum value=$newfilenum>


<table border = 0 width="100%"  rows=1 cols=3 cellpadding=0 cellspacing=2> 
<tr> <!--row that contains entire outer table-->
<td align=center valign=top> <!--outer table left cell (exclude options;doesn\'t have width in older files-->
<table border=0 cellspacing=5 width=\"100%\"> <!-- exclude SSE options-->
 <tr>
  <th class="smalltableheader">Exclude</th>
  <th class="smalltableheader">Residues</th>
  <th class="smalltableheader">Conservation</th>
  <th class="smalltableheader">Type</th>
  <th class="smalltableheader">Blink</th>
 </tr>
END
  for($i=0;$i<=$#query_SSE_starts;$i++) {
    if($excluded{$query_SSE_starts[$i]}) {
      $spanopen = "<span class=\"greyedOut\">";
      $spanclose= "</span>";

      print iter " <tr><td align=center><input type=checkbox name=$query_SSE_starts[$i] checked onClick=\"javascript:checkboxError(this)\"></td>\n";
    }else {
      $spanopen="<span class=\"smalltablefont\">";
      $spanclose="</span>";
      print iter " <tr><td align=center><input type=checkbox name=$query_SSE_starts[$i] onClick=\"javascript:setExcludeString()\"></td>\n";
    }
    print iter "    <td align=center>$spanopen$query_SSE_starts[$i]-$query_SSE_ends[$i]$spanclose</td>\n";
    if(!$excluded{$query_SSE_starts[$i]}) {
      printf iter ("    <td align=center>%s%4.2f%s</td>\n",$spanopen,$conservation{$query_SSE_starts[$i]},$spanclose); 
    }else {
      printf iter ("    <td align=center>%s -- %s</td>\n",$spanopen,$spanclose); 
    }

    if($query_SSE_types[$i] eq "H") {

      printf iter ("    <td>%sHelix%s</td>\n",$spanopen,$spanclose);
    }else {
      printf iter ("    <td>%sStrand%s</td>\n", $spanopen, $spanclose);
    }
    print iter " <td align=center><embed type=\"application/x-spt\"
                              width=12
                              height=12
                              button=push
                              target=\"globalquery\"
                              script=\"select ($query_SSE_starts[$i]-$query_SSE_ends[$i] and not hetero); cartoons off; delay 0.5; cartoons on; delay 0.5; cartoons off; delay 0.5; cartoons on; delay 0.5; cartoons off; delay 0.5; cartoons on\"></td></tr>\n";

  }
  print iter "</table> <!--end SSE exclude table-->\n";
print iter "</td> <!--end of outer table cell (left) containing SSE table-->\n";
print iter "</tr>\n";
print iter "<tr><td>\n";
print iter "<A name=\"reanalyze\"><p><b>To reanalyze your results using different search parameters:&nbsp;</b>\n";
print iter "If you do not exclude any secondary structure elements, your results will be reanalyzed using the search parameters you specify. (Leave all checkboxes above unchecked and specify your search parameters below.)\n";
print iter "</td></tr>\n";
print iter "<tr>\n";
print iter "<td valign=top>\n"; #doesn't have width in older files

print iter "<table border = 0 width=\"100%\"   cellpadding=5 align=left>\n"; #this table has no ending?
print iter "<tr><td align=center colspan=4>\n";
print iter<<END;
<span class=\"smallFrameTitle\">Structural Similarity Search Options</span>
</td></tr>
<tr> <!-- expectation and local vs. global (alignment scope) labels -->
<td colspan=2 align=center><a href=\"Javascript:launchHelpWindow('expectation')\" onMouseOver=\"status='Click for help';return true\" onMouseOut=\"status='';return true\" class="smalloption">Expectation</a></td>
<td colspan=2 align=center><a href="Javascript:launchHelpWindow('alignmentscope')" onMouseOver="status='Click for help';return true" onMouseOut="status='';return true" class="smalloption">Alignment Scope</a></td>
</tr> <!-- end table headers -->

<tr align=center> <!-- expectation text box, global vs. local options -->

<td colspan=2>
<table border=0 align=center valign=center><tr><td><input type=text name=base size=5 value=$base></td><td>e<input type=text name=exponent size=3 value=$exponent></td></tr></table>
</td>

<td rowspan=2 colspan=2>
<table border=0>
<tr><td align=left><input type=radio name=global checked value=1>Global</td></tr>
<tr><td align=left><input type=radio name=global value=0>Local</td></tr>
</table>
</td>
</tr>

<tr align=center> <!-- expectation reset to default button -->
<td colspan=2><input type=button name=defaultExpectationButton value="Reset to Default" onClick="document.myform.exponent.value=1;document.myform.base.value=1.0"></td>
</tr>
</table> <!--end search options table-->
<br clear="all">
<br>

<table border=0 width="100%">
<tr> <!--row for xval-->
<td rowspan=2 align=center><a href="Javascript:launchHelpWindow('conservation')" onMouseOver="status='Click for help';return true" onMouseOut="status='';return true" class="smalloption">Conservation Profile\'s Contribution to Recalculated SSE Scores:</a></td>
<td><table border=0 align=center valign=center><tr><td><input type=text name=xval size=4 value=0.75></td><td>(Value must lie on the interval [0,1])</td></tr></table></td>
</tr>
<tr><td align=center><input type=button name=defaultXvalButton value="Reset to Default" onClick="document.myform.xval.value=0.75"></td></tr>
</table>
<br clear="all">

<table align=left border=0 cellpadding=5 width=\"100%\">

<tr>
 <td>$diststring</td>
</tr>
<tr align=center>
 <td align=center>Use background distribution for: $distselect</td>
</tr>
<tr align=left><td align=left><input type=submit name=submit value=\"Submit\"></tr></td>
</table><br clear=\"all\">
</td> <!--end search options table cell-->
</tr> <!--end outer table row-->
</table>  <!--end outer table-->
</form>
<br clear=\"all\">
</body>
</html>
END

close(iter);
}


#Given the conservation of an SSE, returns a color in rgb format. High conservations
#correspond to bright colors. The color scheme was developed in HSV colorspace with the
#brightest color corresponding to (353,95,80) and the darkest to (353,95,10). 
#Photoshop was used for RGB conversions. Colors are not necessarily "web safe," but
#should print on most printers.
sub getColor {
  my($c) = @_; #conservation

  if($c < 0.05) {
    return((26,1,4));
  }elsif($c < 0.1) {
    return((28,1,5));
  }elsif($c < 0.15) {
    return((31,2,5));
  }elsif($c < 0.2) {
    return((36,2,6));
  }elsif($c < 0.25) {
    return((38,2,6));
  }elsif($c < 0.3) {
    return((43,2,7));
  }elsif($c < 0.35) {
    return((48,2,8));
  }elsif($c < 0.4) {
    return((56,3,9));
  }elsif($c < 0.45) {
    return((61,3,10));
  }elsif($c < 0.5) {
    return((69,3,11));
  }elsif($c < 0.55) {
    return((77,4,12));
  }elsif($c < 0.6) {
    return((84,4,14));
  }elsif($c < 0.65) {
    return((94,5,15));
  }elsif($c < 0.7) {
    return((102,5,16));
  }elsif($c < 0.75) {
    return((117,6,19));
  }elsif($c < 0.8) {
    return((133,7,21));
  }elsif($c < 0.85) {
    return((148,7,24));
  }elsif($c < 0.9) {
    return((184,9,30));
  }else {
    return((204,10,33));
  }

}

#the main results page that includes the three frames
sub printResultsPage {
  my($filenum,$searchiter, $xval, $expectation,$indexframecode,$indexscript,$indexframecode2,$reload_motif_page) = @_;
  my($newfilenum,$motif_file, $table_file, $iterate_file);
  my($outfile, @lines);
 
  $motif_file = "motif$filenum\.html";
  $table_file = "table$filenum\.html";
  $iterate_file = "iterate$filenum\.html";


  open(mainhtml, ">$resultsdir/results.html") || print STDERR "couldn't open $resultsdir/results.html: $!\n";
  $outfile = *mainhtml;

  
print $outfile <<END;
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html>
<head><title>Core Fold/Motif</title>
<script language="Javascript" type="text/javascript">
var controlwin;
var tablewin;

  function reanalyzeOptionsToFront() {
     iterate.location.replace('$outputdir/iterate$filenum.html#reanalyze');
     focus();
  }

  function bringTableToFront() {
    if(tablewin.closed) {
       openTable();
    }
    tablewin.focus();
  }

  function bringControlToFront() {
     if(controlwin.closed) {
       openControlPanel();
     }
     controlwin.focus();
   }

  function openWindows() {
      motif.location.reload(true);
      iterate.location.reload(true);
      openTable();
      openControlPanel(); 
      window.focus();
      controlwin.focus(); //want this in front and motif window right behind it
      if(!tablewin) {
        alert('Please disable popup blockers when viewing this site.');
      }
  }

 //openTable: Opens a table displaying search results. Note that netscape 4.7 (and possibly other versions)
  //do not seem to correctly implement window.open if the page to be displayed is not in the browser\'s
  //cache. Therefore, we open a blank page and replace its URL with the table\'s
  function openTable() {
    tablewin=window.open("", 'table$id');                //open blank page
    tablewin.location.replace('$outputdir/$table_file'); //load table into page 
  }

  function openControlPanel() {
    if(controlwin == null || controlwin.closed) {
       controlwin=window.open("", 'control',"status=yes,resizable=yes,scrollbars=yes,location=no,toolbar=no,menubar=no,directories=no,height=300,width=300");
       controlwin.location.replace("$outputdir/index.html");
    }
    writeControlPanel();
  }
  
  function replaceTable(URL) {
    tablewin.location.replace(URL);
  }

   function writeControlPanel() {
      controlwin.document.open("text/html","replace");
      controlwin.document.writeln($indexframecode);
      controlwin.document.writeln($indexscript);
      controlwin.document.writeln($indexframecode2);
      controlwin.document.close();
   }
END

print $outfile <<END;
   var reload_motif_window = $reload_motif_page
   function checkMotifWindowReloadStatus() {
      if(reload_motif_window == 1) {
           motif.location.reload(true) 
      }
   }

</script>
</head>
<frameset cols="0,100%"  border=0 onLoad="javascript:openWindows()">
   <frame  src="/FoldMiner2/empty.html" name="invisible">
   <frameset cols="350,*" border=1>
     <frame frameborder=1 src="$outputdir/$motif_file" name="motif">
     <frame frameborder=1 src="$outputdir/$iterate_file" name="iterate">
   </frameset>
</frameset>
</html>
END
  close($outfile);
  if($sendresponse) {
    open(IN, "$resultsdir/results.html") || print STDERR "couldn't open $resultsdir/results.html: $!\n";
    @lines = <IN>;
    close(IN);
    print @lines;
  }
}

sub printIndex {
  my($filenum,$searchiter, $xval,$expectation) = @_;
  my($newfilenum,@lines, $i,$j, @fields);
  my($printnewline, $start, $end, $iter, $oldexpect, $oldxval);
  my($indexframecode, $oldquote, $newquote, $oldnewline, $newnewline, $oldslash, $newslash, $selectline);
  my($table_file);
  my($indexscript);
  my($scope,$oldscope);

  if($global) {
    $scope = "Global";
  }else {
    $scope = "Local";
  }

  $newfilenum = $filenum+1;
  $table_file = "table$filenum";
  $indexframecode = $indexscript= "";
  $oldquote = "\"";
  $newquote = "\\\"";
  $oldnewline = "\n";
  $newnewline = "\\n";
  $oldslash = "\/";
  $newslash = "\\/";
  $selectline=-1;
  if($sendresponse) {
    open(oldindex, "$resultsdir/index.html");
    $#lines = -1;
    while($line = <oldindex>) {
      if($line =~ /javascript:loadSearch/) {
	chomp($line);
	$lines[$#lines+1] = $line;
      }
    }
    close(oldindex);
  }

  open(indexfile, ">$resultsdir/index.html") || die "can't open $resultsdir/index.html: $!\n";
  

  #the variable $indexframecode will hold the html for the head portion of the control panel
  $indexframecode = "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">\n";
  $indexframecode.="<html>\n<head>\n<meta http-equiv=\"no-cache\">\n<title>FoldMiner Control Panel</title>\n";
  $indexframecode.="<link rel=stylesheet href=\"/FoldMiner2/lock.css\" type=\"text/css\">\n";
  $indexframecode.= "</head>\n";  

  #the variable $indexscript will hold the javascript code for the control panel
  $indexscript.= "<script language=\"Javascript\" type=\"text/javascript\">\n";
  $indexscript.=  "var tablewin;\n";
  $indexscript.="  var dir=\"$outputdir\";\n";
  $indexscript.=  "function openTable() {\n";
  $indexscript.="   tablewin=window.open(\"\", \'table\');\n";
  $indexscript.= "  tablewin.location.replace(\"$outputdir/$table_file\");\n";
  $indexscript.=    "tablewin.opener.focus();\n";
  $indexscript.="  }\n";
  $indexscript.=   "function bringTableToFront() {\n";
  $indexscript.= "   window.opener.top.bringTableToFront();\n";
  $indexscript.=  "}\n\n";
  $indexscript.=   "function bringMotifToFront(filenum) {\n";
  $indexscript.= "   var parentwin;\n";
  $indexscript.= "   var motifstr;\n";
  $indexscript .= "    var filenumstr = filenum.toString()\n";
  $indexscript.= "   if(window.opener.closed) {\n";
  $indexscript.= "      motifstr = dir+'/motif'+filenumstr+'\.html';\n";
  $indexscript.= "      parentwin = window.open('','motif')\n";
  $indexscript.= "      parentwin.location.replace(motifstr)\n";
  $indexscript.= "   }\n";
  $indexscript.= "   window.opener.focus();\n";
  $indexscript.=  "}\n\n";
  $indexscript.= " function launchHelpWindow(anchor) {\n";
  $indexscript.= "   window.opener.motif.launchHelpWindow(anchor);\n";
  $indexscript.= " }\n";
  $indexscript .= "  function loadSearch(filenum) {\n";
  $indexscript .= "    var filenumstr = filenum.toString()\n";
  $indexscript.= "   var tablestr = dir+'/table'+filenumstr+'\.html';\n";
  $indexscript.= "   var motifstr = dir+'/motif'+filenumstr+'\.html';\n";
  $indexscript.= "   var iterstr = dir+'/iterate'+filenumstr+'\.html';\n";
  $indexscript.= "   window.opener.motif.location.replace(motifstr);\n";
  $indexscript.=  "   window.opener.iterate.location.replace(iterstr);\n";
  $indexscript.= "   window.opener.replaceTable(tablestr);\n";
  $indexscript.= "   window.opener.focus();\n";
  $indexscript.= "   window.focus();\n";
  $indexscript.= " }\n";
  $indexscript.= "</script>\n";



  #the variable $indexframecode will hold the html for the body portion of the control panel
  $indexframecode2.= "<body bgcolor=\"#FFFFFF\" link=\"#26256b\" alink=\"#877bab\" vlink=\"#877bab\">\n";
  $indexframecode2.="<h3>FoldMiner Control Panel</h3>\n";
  $indexframecode2.="<form name=\"frontform\">\n";
  $indexframecode2.="Bring to front:</br>\n";
  $indexframecode2.="<table border=0 cellspacing=5>\n<tr>\n";
  $indexframecode2.="  <td><input type=button name=tablebutton value=\"Table\" onClick=\"javascript:bringTableToFront()\"></td>\n";
  $indexframecode2.="  <td><input type=button name=motifbutton value=\"Motif\" onClick=\"javascript:bringMotifToFront($filenum)\"></td>\n</tr>\n</table>\n";
  $indexframecode2.="<hr>\n";
$indexframecode2.="<h4><a href=\"javascript:launchHelpWindow(\'History\')\" onMouseOver=\"status=\'Click for help\'\" onMouseOut=\"status=\'\'\" class=\"helplink\">History</a></h4>\n";
$indexframecode2.="</form>\n";
$indexframecode2.="<form name=displayform>\n";
$indexframecode2.="<table border=0 cellspacing=2>\n";
$indexframecode2.="<tr align=center>\n";
$indexframecode2.="    <td><a href=\"javascript:launchHelpWindow('display')\" onMouseOver=\"status='Click for help';return true\" onMouseOut=\"status='';return true\" class=\"smallhelplink\">Display<\/a><\/td>\n";
$indexframecode2.="    <td><a href=\"javascript:launchHelpWindow('iterativesearch')\" onMouseOver=\"status='Click for help';return true\" onMouseOut=\"status='';return true\" class=\"smallhelplink\">Iteration<\/a><\/td>\n";
$indexframecode2.="    <td><a href=\"javascript:launchHelpWindow('expectation')\" onMouseOver=\"status='Click for help';return true\" onMouseOut=\"status='';return true\" class=\"smallhelplink\">Expectation<\/a><\/td>\n";
$indexframecode2.="    <td><a href=\"javascript:launchHelpWindow('xval')\" onMouseOver=\"status='Click for help';return true\" onMouseOut=\"status='';return true\" class=\"smallhelplink\">x<\/a><\/td>\n";
$indexframecode2.="    <td><a href=\"javascript:launchHelpWindow('alignmentscope')\" onMouseOver=\"status='Click for help';return true\" onMouseOut=\"status='';return true\" class=\"smallhelplink\">Alignment Scope</a></td>\n";
$indexframecode2 .="<\/tr>\n";


  $printnewline = 1;
  $start = $end = 0;

  # if this is not the first time the control panel has been
  # generated, find a logical place to insert the information 
  # for this analysis of the data (expecatation, xval)
  if($sendresponse) { #this is NOT the first time this page has been generated
    $start = 0;
    $end = $#lines;

    for($i=$start;$i<=$#lines;$i++) {
      @fields = split(/\s+/,$lines[$i]);
#      $iter = $fields[$#fields-6];
#      $oldxval= $fields[$#fields-1];
#      $oldexpect=$fields[$#fields-4];
      $oldscope = $fields[$#fields-1];
      $oldxval=$fields[$#fields-3];
      $oldexpect = $fields[$#fields-5];
      $iter = $fields[$#fields-7];      
      if($iter == $searchiter && $oldexpect == $expectation) {
	#find place to insert based on xval

	if($xval < $oldxval) {
	  $selectline=$i;
          $printnewline=1;
	  $start=0; $end=$i-1; #print new line before this one
	}elsif($xval == $oldxval) { #this search has been done before
	  $printnewline=0;
	  $selectline=$i;
	  $start = 0; $end = $#lines;
	  $lines[$i] =~ s/display onClick/display checked onClick/; #select this line
	}else {  #new xval is larger than this line's xval
	  for($j=0;$j<=$#lines;$j++) {
	    chomp($lines[$j]);
	    @fields = split(/\s+/,$lines[$j]);
	    if($xval < $fields[$#fields-1] && $searchiter == $fields[$#fields-6]) {
	      $start =0; $end= $j-1;
	    }
	  }
	}
	last;
      }elsif($iter > $searchiter) { 
	$start = 0; $end = $i-1;
	last;
      }elsif($iter == $searchiter) {
	if($fields[$#fields-1] > $xval) {
	  $start = 0; $end = $i-1;
	  last;
	}else {
	  for($j=$i+1;$j<=$#lines;$j++) {
	    chomp($lines[$j]);
	    @fields = split(/\s+/,$lines[$j]);
	    if($fields[$#fields-4] > $xval && $iter == $searchiter) {
	      $start = 0; $end = $j-1;
	    }
	  }
	  
	}
      }
    }

    for($i=$start;$i<=$#lines;$i++) {
      if($lines[$i] =~ /^<\/body>$/) {
	last;
      }
      
      $lines[$i] =~ s/display checked onClick/display onClick/; #deselect this radio box
      $lines[$i] =~ s/display checked onClick/display onClick/;
      $indexframecode2.="$lines[$i]\n";
    }
    

    $indexframecode2 .= "<tr align=center><td><input type=radio name=display checked onClick=\"javascript:loadSearch($filenum, $xval,$expectation)\"></td><td><span class=\"smallfont\"> $searchiter <\/span><\/td><td><span class=\"smallfont\"> $expectation </span></td><td><span class=\"smallfont\"> $xval <\/span><\/td><td><span class=\"smallfont\"> $scope </span></td><\/tr>\n";
    $indexframecode2 .= "</table>\n";
  }else {
    $indexframecode2 .= "<tr align=center><td><input type=radio name=display checked onClick=\"javascript:loadSearch($filenum, $xval,$expectation)\"><\/td><td><span class=\"smallfont\"> $searchiter <\/span><\/td><td><span class=\"smallfont\"> $expectation </span></td><td><span class=\"smallfont\"> $xval <\/span><\/td><td><span class=\"smallfont\"> $scope </span></td><\/tr>\n";
    $indexframecode2 .= "<\/table>\n";
  }
  $indexframecode2 .= "<\/form>\n";
  $indexframecode2 .= "<\/body>\n<\/html>";
  print indexfile $indexframecode;
  print indexfile $indexscript;
  print indexfile $indexframecode2;
  $indexframecode =~ s/$oldquote/$newquote/g;   #make sure escaped quotes print correctly
  $indexframecode =~ s/$oldnewline/$newnewline/g; #make sure escaped newlines print correctly
  $indexframecode =~ s/$oldslash/$newslash/g; #escape the '/' character
  $indexframecode = "\"".$indexframecode;
  $indexframecode.="\"";
  $indexframecode2 =~ s/$oldquote/$newquote/g;   #make sure escaped quotes print correctly
  $indexframecode2 =~ s/$oldnewline/$newnewline/g; #make sure escaped newlines print correctly
  $indexframecode2 =~ s/$oldslash/$newslash/g; #escape the '/' character
  $indexframecode2 = "\"".$indexframecode2;
  $indexframecode2 .= "\"";
  $indexscript =~ s/$oldquote/$newquote/g;   #make sure escaped quotes print correctly
  $indexscript =~ s/$oldnewline/$newnewline/g; #make sure escaped newlines print correctly
  $indexscript =~ s/$oldslash/$newslash/g; #escape the '/' character
  $indexscript = "\"".$indexscript;
  $indexscript.="\"";
  return($indexframecode,$indexscript,$indexframecode2);
}

#Need to print out new table.html and new search.sort: use ROC code stuff
sub printSearchTable {
  my($filenum,$searchiter,$xval,$expectation, $sort, $chimesize) = @_;
  my($newfilenum,@orderbyscore);
  my($i);
  my(@errors, @lines, $errorString);
  my($shortscopname);
  my(@foldresults, $target, $fold, $superfam, $fam);
  my($tr, $class, $gray);
  my(@suggestions);
  my($domain);
  my(@targets);

  $shortscopname = $globalquery;
  if($globalquery =~ /^(.*)\.(.*)$/) { #remove filename extensions
    $shortscopname = $1;
  }
  $newfilenum = $filenum+1;
  if($globalquery =~ /queryfile/) {
    $shortscopname = "query";
  }

  #find the most common folds, superfams, etc.
  %foldcounts = ();
  foreach $target (keys(%dp_scores_max)) {
    ($class, $fold, $superfam, $fam) = split(/\./,$classifications{$target});
    $fold = $class.".".$fold;
    $foldcounts{$fold}++;
  }
  @foldresults = sort(sort_by_count keys(%foldcounts));

  #find the most common descriptions (PDB headers)
  %descriptioncounts = ();
  foreach $target (keys(%dp_scores_max)) {
    $descriptioncounts{$description{$target}}++;
  }



  if($sort == -1 || $sort == 2) {   
    @orderbyscore = keys(%dp_scores_max);
    @orderbyscore = sort(sort_by_score keys(%dp_scores_max));
  }elsif($sort != -1) {
    if($sort == 0) { #sort by scop fold (most common folds appear first)
      @orderbyscore = sort(sort_by_fold keys(%dp_scores_max));
    }if($sort == 3) { #sort by statistical significance values (p values)
      @orderbyscore = sort(sort_by_pval keys(%pvalues_max));
    }elsif($sort == 4) { #sort by number of aligned SSEs
      @orderbyscore = sort(sort_by_SSEs keys(%num_aligned_SSEs));
    }elsif($sort == 5) { #sort by number of aligned residues
      @orderbyscore = sort(sort_by_residues keys(%num_aligned_residues));
    }elsif($sort == 6) { #sort by rmsd
      @orderbyscore = sort(sort_by_rmsd keys(%RMSD));
    }elsif($sort == 7) { #sort by pdb header (most common headers appear first)
      @orderbyscore = sort(sort_by_description keys(%description));
    }
  }

  #print the results to a text file for download (if desired by user)
  printTextFileResults(\@orderbyscore, $sort, $filenum,"search.sort");
  @targets = keys(%SSE_scores);
  printTextFileResults(\@targets, 0, $filenum, "search.sort.all");

  open(searchtable, ">$resultsdir/table$filenum\.html") || die "can't open $resultsdir/table$filenum\.html for writing: $!\n";
  printSearchTableHeader(*searchtable);

  #find errors (such as files that were not found or that could not be read)
  if($sortcol==-1) {
    open(LOG, "$resultsdir/errorlog");
    @lines = <LOG>;
    close(LOG);
    @errors = map(/^Sorry\.  (.*)/,@lines); #this takes a long time!
    printErrorFile(\@errors);
  }

  #print the search table
  print searchtable "<table border=0 cellpadding=2>\n";
  print searchtable "<tr>\n";
  print searchtable "<td><span class=smallfont>View: </span></td>\n";
  print searchtable "<td><a href=\"#folds\">SCOP fold statistics</a></td>\n";
  print searchtable "<td><a href=logfile class=smallfont>Execution Log (very large file!)</a></td>\n";
  print searchtable "<td><a href=\"$outputdir/search\.sort$filenum\" class=smallfont>Search results (non-html)</a></td>\n";
  print searchtable "</tr>\n</table>\n";
    print searchtable "<br clear=\"all\">\n";
  print searchtable "<i><font size=-1>Too many results? If so, try reducing the <a href=\"javascript:launchHelpWindow('xval')\" onMouseOver=\"status='Click for help';return true\" onMouseOut=\"status='';return true;\" class=\"helplink\">conservation profile's contribution to recalculated SSE scores</a> from";
  if($xval == 0.75) {
      print searchtable " 0.75 to 0.5 ";
  }elsif($xval == 0.5) {
      print searchtable " 0.5 to 0.25 ";
  }elsif($xval == 0.25) {
      print searchtable " 0.25 to 0 ";
  }
  print searchtable "<a href=\"javascript:bringMotifToFront()\">here</a>.";
  print searchtable "</i></font><br>\n";  


  #print the column headers
  printTableHeaders(*searchtable,$chimesize);

  #print a line in the table for each hit
  for($i=0;$i<=$#orderbyscore;$i++) {
    printTableLine($shortscopname,$orderbyscore[$i],$i+1, *searchtable,$filenum);
  }
  print searchtable "</table>\n";

  if($#orderbyscore > -1 && !($#orderbyscore==0 && $orderbyscore[0] eq $shortscopname)) {
      #print fold statistics
      print searchtable "<br><a name=\"folds\"></a><table border = 0>\n";
      print searchtable "<tr bgcolor=\"#bbbbbb\"><th align=left><a href=\"javascript:launchHelpWindow('selectfold')\" onMouseOver=\"status='Click for help';return true\" onMouseOut=\"status='';return true;\" class=\"helplink\">Select</a></th><th align=left>SCOP Fold</th><th align=left>Count</th></tr>\n";

      $gray = 0;
      foreach $fold (@foldresults) {
	  if($gray) {
	      $tr = "<tr bgcolor=\"#eeeeee\">";
	      $gray = 0;
	  }else {
	      $tr = "<tr>";
	      $gray = 1;
	  }
	  print searchtable "$tr\n<td align=left><input type=checkbox name=$fold onClick=\"javascript:colorFold()\"></td><td align=left><a href=\"http://scop.berkeley.edu/search.cgi?sccs=$fold\" target=_blank>$foldnames{$fold}</a></td><td align=right><span class=\"searchInfo\">$foldcounts{$fold}</span></td></tr>\n";

      }
      print searchtable "</table>\n";
      @suggestions = findDomainSuggestions($globalquery);
  }else {
    @suggestions = findDomainSuggestions($globalquery);
    print searchtable "<br>\n";
    print searchtable <<END;
There are no statistically significant alignments. You may want to try one of the following options:
   <ul>
     <li> Increase the <a href=\"javascript:launchHelpWindow('expectation')\" onMouseOver=\"status='Click for help';return true\" onMouseOut=\"status='';return true;\" class=\"helplink\">expectation</a> value. This can be done <a href=\"javascript:bringMotifToFront()\">here</a> without performing additional superpositions.
     <li> Search for local alignments by changing the <a href=\"javascript:launchHelpWindow('alignmentscope')\" onMouseOver=\"status='Click for help';return true\" onMouseOut=\"status='';return true;\" class=\"helplink\">alignment scope</a>. This can be done <a href=\"javascript:bringMotifToFront()\">here</a> without performing additional superpositions.
END
  if($#suggestions > 0) {

    print searchtable "     <li> FoldMiner is primarily designed to favor global alignments. If your query structure has multiple domains, run a FoldMiner search for each domain separately.<br><br>The query you entered is divided into more than one domain in the <a href=\"http://scop.berkeley.edu/\">SCOP</a> hierarchy: Each of these SCOP identifiers may be entered as a query on the FoldMiner homepage\n";
    print searchtable "           <ul>\n";

    for $domain (@suggestions) {
      print searchtable "             <li><a href=\"http://scop.berkeley.edu/search.cgi?sid=$domain\">$domain</a>\n";
    }
    print searchtable "           </ul>\n";
  }else {
    print searchtable "     <li> FoldMiner is primarily designed to favor global alignments. If your query structure has multiple domains, run a FoldMiner search for each domain separately.\n";

  }
    print searchtable "</ul>\n";
}

  print searchtable "</form>\n";
  print searchtable "<\/body>\n</html>\n";
  close(searchtable);
  if($sort != -1) { #need to print the file to STDOUT as the cgi response. Better way to do this??
    print "Content-type: text/html\n\n";
    open(IN, "$resultsdir/table$filenum\.html") || die "can't open $resultsdir/table$filenum\html: $!\n";
    @lines = <IN>;
    close(IN);

    print "@lines";
  }
}
    
sub printErrorFile {
  my($errors) = @_;
  
  open(OUT, ">$resultsdir/errors.html");

  print OUT <<END;
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html>
<head>
<title>FoldMiner Search Errors</title>
</head>
<body>
<h1>Errors</h1>
<p>The following errors occurred during your search. If a pdb or dssp file for a target was not found or did not appear to be formatted correctly, FoldMiner skipped the target and continued.<br>
<UL>
END


  for($i=0;$i<=$#$errors;$i++) {
    print OUT "<li>$$errors[$i]\n";
  }
  print OUT "</UL>\n</body></html>";
  
}

#This subroutine prints out information about a hit to a line in the searh
#results table
sub printTableLine {
  my($query,$target,$target_rank, $file,$filenum) = @_;
  my($shortpdbname,$shortscopname,$targetscoplink);
  my($score,$pvalue,$description,$rmsd);
  my($url, $yourfile);


  #So far, all search databases consist of SCOP domains
  ($shortpdbname) = $target =~ /^.(....)/;
  ($shortscopname) = $target =~/^(.......)/; #remove any filename extensions

  $targetscoplink = "http://scop.berkeley.edu/search.cgi?sid=$shortscopname";
  print $file "<tr>\n";
  print $file "<td align=left>$target_rank&nbsp;</td>\n";
  print $file "<td align=left>";
  print $file "<a href=$targetscoplink target=$shortscopname>$target</a></td>\n";
  print $file  "<td align=center>";
  print $file  "<a href=$outputdir/$query\_$target\.out>Text</a>";
  print $file  " ,\n ";
  print $file "<a href=/cgi-bin/FoldMiner2/newpdb.cgi?yourfile=$query\_$shortscopname\.out\&outfile=$query\_$shortscopname\.pdb\&outdir=$resultsdir\&wwwdir=$outputdir\&chime=0>PDB</a>";
  print $file " ,\n ";
  $score = sprintf("%4.2f",$finalscores{$target});
  $pvalue = sprintf("%1.2g",$pvalues_max{$target});
  $rmsd = sprintf("%4.2f",$RMSD{$target});
  $description = CGI::escape($description{$target});
  $yourfile = "$query\_$shortscopname\.out";

  print $file  "<a href=\"javascript:viewAlignment('/cgi-bin/FoldMiner2/newpdb.cgi?yourfile=$query\_$shortscopname\.out\&outfile=$query\_$shortscopname\.pdb\&outdir=$resultsdir\&wwwdir=$outputdir\&chime=1\&motif=1\&search=1\&filenum=$filenum&score=$score&pval=$pvalue&alignedSSEs=$num_aligned_SSEs{$target}&alignedresidues=$num_aligned_residues{$target}&rmsd=$rmsd&description=$description')\" onMouseOver=\"status='View alignment'\" onMouseOut=\"status=''\">Chime</a>";
  print $file "</td>\n";

  #alignment info 
  printf $file ("<td align=center>%4.2f</td>\n",$finalscores{$target}); #score
  printf $file ("<td align=center>%1.2g</td>\n",$pvalues_max{$target}); #p value
  print $file "<td align=center>$num_aligned_SSEs{$target}</td>\n"; #num aligned SSEs
  print $file "<td align=center>$num_aligned_residues{$target}</td>\n"; #number of aligned residues
  print $file "<td align=center>$RMSD{$target}</td>\n"; #RMSD
  print $file "<td align=left><font size=-1>$description{$target}</font></td>\n"; #PDB header
  print $file "</tr>\n";
}

#this subroutine prints the column headers for the search results table
sub printTableHeaders {
  my($file,$chimesize) = @_;

  print $file "<form name=\"sortform\" action=\"/cgi-bin/FoldMiner2/formatSearchTable.cgi\">\n<input type=hidden name=sortcol value=2>\n";
  print $file "<input type=hidden name=xval value=$xval>\n";
  print $file "<input type=hidden name=iter value=$searchiter>\n";
  print $file "<input type=hidden name=resultsdir value=\"$outputdir\">\n";
  print $file "<input type=hidden name=query value=$globalquery>\n";
  print $file "<input type=hidden name=querylink value=\"$querywwwlink\">\n";
  print $file "<input type=hidden name=chime value=$chime>\n";
  print $file "<input type=hidden name=filenum value=$filenum>\n";
  print $file "<input type=hidden name=chimesize value=$chimesize>\n";
  print $file "<input type=hidden name=selectedfolds value=''>\n";
  print $file "<table width=\"100%\" cellpadding=0 cellspacing=0 border=1>\n";
  print $file " <tr>\n";

  print $file "       <th align=left valign=top>&nbsp;<br><br><br>\n";
  print $file "       <th align=center valign=top><b>Target</b><br><br><a href=\"javascript:sortTable(0)\" class=\"tablesort\">Sort by fold</a></th>\n";
  print $file "       <th align=center valign=top><b><a href=\"javascript:launchHelpWindow('resultsformats')\" onMouseOver=\"status='Click for help';return true\" onMouseOut=\"status='';return true\" class=\"helplink\">View Results</a></b><br><br></th>\n";
  print $file "       <th align=center valign=top><b><a href=\"javascript:launchHelpWindow('alignmentscore')\" onMouseOver=\"status='Click for help';return true\" onMouseOut=\"status='';return true\" class=\"helplink\">Score</a></b><br><br><a href=\"javascript:sortTable(2)\" class=\"tablesort\">Sort</a></th>\n";
  print $file "       <th align=center valign=top><b><a href=\"javascript:launchHelpWindow('significance')\" onMouseOver=\"status='Click for help';return true\" onMouseOut=\"status='';return true\" class=\"helplink\">P value</a></b><br><br><a href=\"javascript:sortTable(3)\" class=\"tablesort\">Sort</a></th>\n";
  print $file "       <th align=center valign=top><b>SSEs</b><br><b>aligned</b><br><a href=\"javascript:sortTable(4)\" class=\"tablesort\">Sort</a></th>\n";
  print $file "       <th align=center valign=top><b>CA Atoms</b><br><b>Aligned</b><br><a href=\"javascript:sortTable(5)\" class=\"tablesort\">Sort</a></th>\n";
  print $file "       <th align=center valign=top><b>RMSD</b><br><br><a href=\"javascript:sortTable(6)\" class=\"tablesort\">Sort</a></th>\n";
  print $file "       <th align=center valign=top><b>PDB HEADER</b><br><br><a href=\"javascript:sortTable(7)\" class=\"tablesort\">Sort</a></th>\n";
  print $file "      </tr>\n";
}


#Prints beginning of the html for the search table page
sub printSearchTableHeader {
  my($file) = @_;
  my($shortscopname);


  if($globalquery =~ /^$resultsdir\/(queryfile)\.pdb$/) {
    $shortscopname = $1;
  }else {
    $shortscopname = $globalquery;
    
    if($globalquery =~/^(.*)\.(.*)$/) { #remove any filename extensions
      $shortscopname = $1;
    }
  }

  print $file "<html>\n";
  print $file  "<head>\n<title>FoldMiner Search Results Table</title>\n<link rel=stylesheet href=\"/FoldMiner2/lock.css\" type=\"text/css\">\n";
  print $file <<END;
  <script language="Javascript">
    var viewerwin;

    function bringMotifToFront() {
       opener.reanalyzeOptionsToFront();
    }

    function launchViewer(yourfile,outdir,wwwdir,chime) {
      var URL = "/cgi-bin/FoldMiner2/newpdb.cgi?";
      URL += "yourfile="+yourfile;
      URL += "&outdir="+outdir;
      URL += "&resultsdir="+wwwdir;
      URL += "&chime="+chime;
      var mywin=window.open(URL, "alignmentviewer", "directories=no,location=no,menubar=no,toolbar=no,scrollbars=yes,resizable=yes,status=yes");
      mywin.focus();
    }
   
    function colorFold() {
       var str = "";
         for(var i=0;i<document.sortform.elements.length;i++) {
           if(document.sortform.elements[i].type == "checkbox") {
             if(document.sortform.elements[i].checked) {
                str += document.sortform.elements[i].name;
                str += ",";
             }
           }
         }
       str = str.substring(0,str.length-1); //remove last comma
       document.sortform.selectedfolds.value=str;
       document.sortform.submit();
    }

    function sortTable(col) {
       document.sortform.sortcol.value=col;
       document.sortform.submit()
    }

    function bringControlToFront() {
       window.opener.top.bringControlToFront();
    }

    function viewAlignment(URL) {
      if(viewerwin == null || viewerwin.closed) {
         viewerwin = window.open('','viewer');
      } 
      viewerwin.location.replace(URL);
      viewerwin.focus();
   }

  function launchHelpWindow(anchor) {
   var URL = "/FoldMiner2/help.html#"+anchor;
   var mywin=window.open('','help',"directories=no,location=no,menubar=no,scrollbars=yes,toolbar=no,resizable=yes,status=yes,width=350,height=300");
   mywin.location.replace(URL);
    mywin.focus();
  }
  </script>
</head>
END

  print $file "<body bgcolor=#FFFFFF link=#26256b alink=#877bab vlink=#877bab>\n";
  print $file "<a href=/FoldMiner2/><img align=left src=/FoldMiner2/images/foldminer.gif border=0></a><br clear=\"all\">\n<br>\n";
  print $file "<b>FoldMiner search results for query: <a href=$querywwwlink target=$shortscopname>$shortscopname</a></b><br>\n";
  print $file "<a href=\"javascript:bringControlToFront()\" class=\"smallfont\">(Bring control panel to front)</a><br>\n";

}

sub findDomainSuggestions {
  my(@subdomains);
  my($domain);
  my($pdbid,$chain);
  my($pdbid2);

  $#subdomains = -1;
  if(length($globalquery) == 4) {
    $pdbid = substr($globalquery,0,4);
    $chain = ".";
  }elsif(length($globalquery)==6) {
    $pdbid = substr($globalquery,0,4);
    $chain = substr($globalquery,5,1);
  }else {
    return(@subdomains);
  }

  for $domain (keys(%classifications)) {
    if($domain =~ /^.$pdbid$chain.$/) {
      $subdomains[$#subdomains+1] = $domain;
    }
  }

#  print STDERR "@subdomains\n";
  return @subdomains;
}

#targets with folds with higher counts (folds which appear more often 
#among statistically significant targets) are "less than" other targets.
#Within the same fold, ties are broken by alignment score.
sub sort_by_count {
  my($fold1, $fold2) = ($a, $b);
  if($foldcounts{$b} == $foldcounts{$a}) {
    $foldnames{$fold1} cmp $foldnames{$fold2};
  }else {
    $foldcounts{$b} <=> $foldcounts{$a};
  }
}

#targets with PDB headers (descriptions) that appear more often among
#statistically significant targets are "less than" other targets. Within
# the same description, ties are broken by alignment score.
sub sort_by_description {
  my($domain1, $domain2) = ($a,$b);
  
  if($description{$domain1} ne $description{$domain2}) {
    $descriptioncounts{$description{$domain2}} <=> $descriptioncounts{$description{$domain1}};
  }else { #sort by score within the same description
    $dp_scores_max{$b} <=> $dp_scores_max{$a};
  }
  
}

#Sorts targets by the folds that appear most often. Within the same fold,
#targets with higher scores appear first.
sub sort_by_fold {
  my($domain1, $domain2) = ($a, $b);
  my($class1,$fold1, $superfam1,$fam1, $class2, $fold2, $superfam2, $fam2);
  my($foldnum1, $foldnum2);

  ($class1,$foldnum1,$superfam1,$fam1) = split(/\./,$classifications{$a});
  ($class2,$foldnum2,$superfam2,$fam2) = split(/\./,$classifications{$b});
  $fold1 = $class1.".".$foldnum1;
  $fold2 = $class2.".".$foldnum2;
  if($fold1 ne $fold2) {
    if($foldcounts{$fold2} == $foldcounts{$fold1}) {
      $foldnames{$fold1} cmp $foldnames{$fold2};
    }else {
      $foldcounts{$fold2} <=> $foldcounts{$fold1};
    }
  }else { #sort by score within same fold
    $dp_scores_max{$b} <=> $dp_scores_max{$a};
  }
  
}


#Sorts folds into a list such that folds in each SCOP class appear together
#in numerical order. The distribution representing all of SCOP appears first,
#and distributions for entire classes appear before distributions for
#individual folds within that class
sub sort_folds {
  my($class1, $class2);
  my($fold1, $fold2);

  #the fold placeholder "all" (representing the overall distribution) is always first
  if($a eq "all") {
    return(-1);
  }elsif($b eq "all") {
    return(1);
  }else {
    ($class1,$fold1) = split(/\./,$a);
    ($class2, $fold2) = split(/\./,$b);
    if($class1 eq $class2) {
      if($fold1 < $fold2) {
	return(-1);
      }elsif($fold2 < $fold1) {
	return(1);
      }else {
	return(0);
      }
    }elsif($class1 lt $class2) {
      return(-1);
    }elsif($class2 lt $class1) {
      return(1);
    }else {
      return(0);
    }
  }

  
}


#The target with the higher score is "less than" the target with the
#lower score
sub sort_by_score {
    $dp_scores_max{$b} <=> $dp_scores_max{$a}
}

#The target with the lower p value is "less than" the target with
#the higher p value
sub sort_by_pval {
  $pvalues_max{$a} <=> $pvalues_max{$b};
}

#The target with more aligned SSEs is "less than" the other
sub sort_by_SSEs {
  $num_aligned_SSEs{$b} <=> $num_aligned_SSEs{$a};
}

#The target with more aligned residues is "less than" the other
sub sort_by_residues {
  $num_aligned_residues{$b} <=> $num_aligned_residues{$a};
}

#The target with the lower rmsd is "less than" the other
sub sort_by_rmsd {
  $RMSD{$a} <=> $RMSD{$b};
}

#Returns the value of the CDF of extreme value distribution for 
#parameters lambda and mu at x
sub expectation_function {
  my($lambda,$mu,$x) = @_;

  return(1-exp(-exp(-$lambda*($x-$mu))));
}

sub inverse_exponential_function {
  my($x, $y) = @_;
  my($retvalue);

  return(-log($y)/$x);
}


sub printTextFileResults {
  my($resultsref,$sorting,$filenum,$basefilename) = @_;
  my($i,$included);
  
  open(OUT, ">$resultsdir/$basefilename$filenum") || die "can't open $resultsdir/search.sort: $!\n";
  for($i=0;$i<=$#$resultsref;$i++) {
    if($sorting==-1) {
      if($pvalues_max{$$resultsref[$i]} <= $pcutoff) {
	$included = 1;
      }else {
	$included = 0;
      }
    }else {
      $included = 1;
    }
    print OUT "$$resultsref[$i]|$finalscores{$$resultsref[$i]}|$pvalues_max{$$resultsref[$i]}|$num_aligned_SSEs{$$resultsref[$i]}|$num_aligned_residues{$$resultsref[$i]}|$RMSD{$$resultsref[$i]}|$description{$$resultsref[$i]}\n";
    
  }
}

#Reads in alignment results
sub readSearchFile {
  my($resultdir,$sorting,$filenum) = @_; #assume dirname has already been checked for correct syntax
  my($file);
  my($line,@fields,$i,$start,$start2,$end,$end2,$score,$sstype);
  my($target);
  my($SSEscorefieldstart);
  my($tclass,$tfold,$tsuperfam,$tfam);

  #check syntax of filenum
  if($filenum =~ /^([0-9]+)$/) {
    $filenum = $1;
  }

  %num_aligned_SSEs = ();
  %finalscores = ();
  %dp_scores_max = ();
  %SSE_scores = ();
  %num_aligned_residues = ();
  %RMSD = ();
  %description = ();

  if($sorting == -1) {
    $file = $resultdir."/search.out";
    $SSEscorefieldstart = 6;
  }else {
    $file = $resultdir."/search.sort$filenum";
    $SSEscorefieldstart = 8;
  }

  open(IN, $file) || die "can't open $file: $!\n";
  while($line = <IN>) {
    last if(length($line) < 4); #shouldn't need this check?
    chomp($line);
    @fields = split(/\|/,$line);
    $target = $fields[0];
    ($tclass,$tfold,$tsuperfam,$tfam) = split(/\./,$classifications{$target});
    next if($tclass !~ /[abcd]/);
    if($sorting == -1) {
      for($i=$SSEscorefieldstart;$i<=$#fields;$i++) {
	($start,$start2,$end,$end2,$score,$sstype) = split(/,\s*/,$fields[$i]);
	$SSE_scores{$target}{$start} = $score;
      }

      $num_aligned_SSEs{$target} = $fields[2];
      $num_aligned_residues{$target} = $fields[3];
      $RMSD{$target} = $fields[4];
      $description{$target} = $fields[5];
      
      $dp_scores_max{$target} = $finalscores{$target} = $fields[1];
    }else {
#      if($fields[1] == 1) {
	$num_aligned_SSEs{$target} = $fields[3];
	$num_aligned_residues{$target} = $fields[4];
	$RMSD{$target} = $fields[5];
	$description{$target} = $fields[6];
	$dp_scores_max{$target}=$finalscores{$target} = $fields[1];
	$pvalues_max{$target} = $fields[2];
#      }
    }
  }
  close(IN);
  @fields = keys(%dp_scores_max);
}

#Prints a page that's displayed while alignments are run.
sub printWaitPage {
  my($filenum,$id, $expectation, $xval,$chime,$targetdb) = @_;
  my($searchiter) = 1;
  my($file, @lines);
  my($numcompleted);
  my($log, $error, $grepstring,$myfile);

  print "Content-type: text/html\n";
  print "Cache-control: no-cache\n\n";

  $filenum = 1; #always 1 if printWaitPage can be reached

  $file = "/var/www/html/FoldMiner2/results/$id/table$filenum\.html";
  $log = "$resultsdir/logfile";
  $grepstring = "\"\\*\\* Target = \" ";


  if(! (-e $file)) {
    open(IN, "grep -c $grepstring $log|");
    $numcompleted = <IN>; #number of alignments that have finished
    close(IN);

    chomp($numcompleted);

    if($numcompleted !~ /[0-9]+/) {
      $numcompleted = 0;
    }
    

    print <<END;
<html>
<head>
<meta http-equiv="Pragma" content='no-cache'>
<meta http-equiv=refresh content=30>
<title>FoldMiner Search Results . . . Processing</title>
<script language="Javascript">
  function refreshPage() {
    window.location.reload();
  }
</script>
</head>
<body>
<h1>FoldMiner Search: In Progress</h1>
<p>Your search is executing. If you entered an email address, you will be notified by email when it is complete. This page will refresh every 30 seconds; results will appear when your search is complete.<br>

<p>You may also click on the following link to check on the progress of your search:</p><br>
<div align=center><a href="javascript:refreshPage()" onMouseOver="status='Check search status'" onMouseOut="status=''">Are my results ready?</a></div>
<p>Number of alignments completed as of last check: $numcompleted out of $databasesize</p><br>
<p><i>(Please note that all targets are aligned to the query structure in a non-random order. The number of alignments completed thus far cannot be used to compute the approximate amount of time required to complete all of the alignments.)</i>
<p>Your results will be available at http://dlb3.stanford.edu/FoldMiner/results/$id/results.html after the search finishes.</p>
<p><b><font color="red">Results may be deleted without warning 72 hours after the FoldMiner run has finished. Please be sure to download results in text format from the search results browser window. A rasmol script describing the structural motif identified by FoldMiner may be downloaded as well.</font></b>
</body>
</html>
END
  }else { #alignments have finished
    open(IN, "/var/www/html/FoldMiner2/results/$id/results.html");
    @lines = <IN>;
    close(IN);
    print @lines;
  }
}

#check to be sure that the filename is actually one of the astral files
#Also sets databasesize since it's extracting the info needed to do that anyway
sub checkTargetDBFilename {
  my($filename) = @_;


  if($filename =~m|^$databasedir/astral-scopdom-seqres-gd-sel-gs-bib|) { #using % identity
    if($filename =~ m|^($databasedir/astral-scopdom-seqres-gd-sel-gs-bib-)([0-9]{2,3})(-$VERSION.id)|) {
      $filename = $1.$2.$3;

      $databasesize = $numdomains{$2};
      $percentID = $2;
      $scopLevel = -1;
    }else {
      $filename = "";
      $percentID = -1;
      $scopLevel = -1;
    }
  }elsif($filename =~m|^($databasedir/astral-scopdom-seqres-gd-sel-gs-sc-)([a-z]{2})(-$VERSION.id)|) {#using hierarchy representatives

    $percentID = -1;
    if(($2 eq "cl" || $2 eq "cf" || $2 eq "sf" || $2 eq "fa" || $2 eq "dm")) {
      $filename = $1.$2.$3;
      $scopLevel = $2;
      $databasesize = $numdomains{$2};
    }else {
      $filename = "";
    }
  }else { #filename not recognzied (not one of the astral files)
    $percentID = -1;
    $scopLevel = -1;
    return "";
  }
  return($filename);
}

sub securityCheckProteinName {
  my($inputname, $isScop) = @_;
  my($cleanname,$pdbid);
  
  $cleanname = "";
  if($isScop) {
    $inputname =~ /^([dg][0-9][A-Za-z0-9]{3}[\._A-Za-z0-9][_0-9])/;
    $cleanname = $1;
  }else {
    $inputname =~/^([0-9][A-Za-z0-9]{3})(-[A-Za-z0-9])?/;
    $cleanname = $1.$2; #$1 is the pdb id, and $2 is the chain, prefixed by '-'
  }
  return $cleanname; #need to do error checking -- could return empty string
}


sub getColorByConservation {
  my($hue, $sat, $interval, $ratio) = @_;
  my($r,$g,$b);
  

  if($interval == 0) {
      ($r, $g, $b) = HSVtoRGB($hue, $sat, 20*($ratio**0));
    }elsif($interval == 1) {
      ($r, $g, $b) = HSVtoRGB($hue, $sat, 20*($ratio**1));
    }elsif($interval == 2) {
      ($r, $g, $b) = HSVtoRGB($hue, $sat, 20*($ratio**2));
    }elsif($interval == 3) {
      ($r, $g, $b) = HSVtoRGB($hue, $sat, 20*($ratio**3));
    }elsif($interval == 4) {
      ($r, $g, $b) = HSVtoRGB($hue, $sat, 20*($ratio**4));
    }elsif($interval == 5) {
      ($r, $g, $b) = HSVtoRGB($hue, $sat, 20*($ratio**5));
    }elsif($interval == 6) {
      ($r, $g, $b) = HSVtoRGB($hue, $sat, 20*($ratio**6));
    }elsif($interval == 7) {
      ($r, $g, $b) = HSVtoRGB($hue, $sat, 20*($ratio**7));
    }elsif($interval == 8) {
      ($r, $g, $b) = HSVtoRGB($hue, $sat, 20*($ratio**8));
    }elsif($interval == 9) {
      ($r, $g, $b) = HSVtoRGB($hue, $sat, 20*($ratio**9));
    }elsif($interval == 10) {
      ($r, $g, $b) = HSVtoRGB($hue, $sat, 20*($ratio**10));
    }elsif($interval == 11) {
      ($r, $g, $b) = HSVtoRGB($hue, $sat, 20*($ratio**11));
    }elsif($interval == 12) {
      ($r, $g, $b) = HSVtoRGB($hue, $sat, 20*($ratio**12));
    }elsif($interval == 13) {
      ($r, $g, $b) = HSVtoRGB($hue, $sat, 20*($ratio**13));
    }elsif($interval == 14) {
      ($r, $g, $b) = HSVtoRGB($hue, $sat, 20*($ratio**14));
    }elsif($interval == 15) {
      ($r, $g, $b) = HSVtoRGB($hue, $sat, 20*($ratio**15));
    }elsif($interval == 16) {
      ($r, $g, $b) = HSVtoRGB($hue, $sat, 20*($ratio**16));
    }elsif($interval == 18) {
      ($r, $g, $b) = HSVtoRGB($hue, $sat, 20*($ratio**17));
    }elsif($interval == 19) {
      ($r, $g, $b) = HSVtoRGB($hue, $sat, 20*($ratio**18));
    }else {
      ($r, $g, $b) = HSVtoRGB($hue, $sat, 20*($ratio**19));
    }
  return($r,$g,$b);
}


#convert an hsv color specification to rgb
sub HSVtoRGB {
  my($hue, $sat, $value) = @_;
  my($r,$g,$b);
  my($i, $f, $p, $q, $t);


  #$sat and $value should lie on the range [0,1]
  $sat /= 100;
  $value /= 100;
  if($sat == 0) {
    $r = $g = $b = ($value*255);
  }else {
    if($hue == 360) { $hue = 0;}
    $hue/=60.0;
    $i = int($hue); #integer part of $floathue
    $f = $hue - $i; #fractional part of $floathue

    $p = $value * (1.0-$sat);
    $q = $value * (1.0 - ($sat*$f));
    $t = $value * (1.0 - ($sat*(1.0-$f)));

    if($i==0) {
      $r = $value * 255;
      $g = $t*255;
      $b = $p*255;
    }elsif($i==1) {
      $r = $q * 255;
      $g = $value*255;
      $b = $p*255;
    }elsif($i==2) {
      $r = $p *255;
      $g = $value*255;
      $b = $t*255;
    }elsif($i==3) {
      $r = $p*255;
      $g = $q*255;
      $b = $value*255;
    }elsif($i==4) {
      $r = $t*255;
      $g = $p*255;
      $b = $value*255;
    }elsif($i==5) {
      $r = $value*255;
      $g = $p*255;
      $b = $q*255;
    }
    
  }
  #round the numbers
  $r = int($r+0.5);
  $g = int($g+0.5);
  $b = int($b+0.5);
  return($r,$g,$b);
}

sub redirectToErrorHandler {
  my($input,$error,$correctinput,$helptopic,$pid) = @_;
  my($email) = "jessicas\@stanford.edu";
  my($name) = "Jessica Ebert";

  $input=CGI::escape($input);
  $error = CGI::escape($error);
  $correctinput=CGI::escape($correctinput);
  $helptopic=CGI::escape($helptopic);
  $email=CGI::escape($email);
  $name=CGI::escape($name);

  print "Cache-control: no-cache\n";
  print "Content-type: text/html\n\n";
  print <<END;
<html>
<head>
<meta http-equiv="Pragma" content='no-cache'>
<meta http-equiv="refresh" content="0;URL=/cgi-bin/FoldMiner2/reporterror.cgi?error=$error&exampleinput=$correctinput&input=$input&email=$email&name=$name&helptopic=$helptopic&resultsdir=$pid&runtype=search">
<title>FoldMiner: Error</title>
</head>
<body>
An error occurred. If you are not automatically redirected in two seconds, <a href="/cgi-bin/FoldMiner2/reporterror.cgi?error=$error&exampleinput=$correctinput&input=$input&email=$email&name=$name&helptopic=$helptopic&resultsdir=$pid&runtype=search">click here.</a>
</body>
</html>
END

}
