#!/usr/bin/perl -T

# Copyright 2003 The Board of Trustees of 
# The Leland Stanford Junior University. All Rights Reserved.

# Communication to Jessica Shapiro, jessicas@stanford.edu


use lib "/var/www/cgi-bin/FoldMiner2";
use CGI;
use Scop qw(%classifications %foldnames);

my($q) = new CGI;

my($query,$outputdir,$searchiter,$chime,$wwwdir,$filenum,$xval,$expectation,$sort,$chimesize,$global,$selectedfold); #parameters
my($ID);

#get parameters
$query = $q->param('query');
$wwwdir = $q->param('resultsdir');
$filenum = $q->param('filenum');
$searchiter = $q->param('iter');
$chime = $q->param('chime');
$xval = $q->param('xval');
$expectation = $q->param('expectation');
$sort = $q->param('sortcol');
$chimesize = $q->param('chimesize');
$global = $q->param('global');
$querywwwlink = $q->param('querylink');
$selectedfolds = $q->param('selectedfolds');

if($wwwdir =~ m|^(/FoldMiner2/results/)([0-9]+)$|) {
  $wwwdir = $1.$2;
  $ID = $2;
  $outputdir = "/var/www/html".$wwwdir;
}else {
    redirectToErrorHandler('',"An internal variable (wwwoutput) has an unexpected value.","","",$ID);exit(-1);
}

if($filenum =~ /^([0-9]+)$/) {
    $filenum = $1;
}else {
    redirectToErrorHandler('',"An internal variable (filenum) has an unexpected value.","","",$ID);
    exit(-1);
}


%color_fold = ();

if(defined($selectedfolds) && $selectedfolds ne "") {
  @fields = split(",",$selectedfolds);
  for($i=0;$i<=$#fields;$i++) {
    $color_fold{$fields[$i]} = 1;
#    print "coloring $fields[$i] $color_fold{$fields[$i]}\n";
  }
}

#need to read in targets and their info
my($file);
$file=$outputdir."/"."search.sort".$filenum;
my(@orderbyscore,%pvalues,%scores,%num_aligned_SSEs,%num_aligned_residues,%RMSD,%description);
my($line,@fields);

open(IN,$file) || print STDERR "can't open $file: $!\n";
$#orderbyscore = -1;
while($line = <IN>) {
  chomp($line);
  @fields = split(/\|/,$line);
  $orderbyscore[$#orderbyscore+1] = $fields[0];
  $scores{$fields[0]} = $fields[1];
  $pvalues{$fields[0]} = $fields[2];
  $num_aligned_SSEs{$fields[0]} = $fields[3];
  $num_aligned_residues{$fields[0]} = $fields[4];
  $RMSD{$fields[0]} = $fields[5];
  if($#fields == 6) {
    $description{$fields[0]} = $fields[6];
  }else {
    $description{$fields[0]} = "";
  }
}


my($newfilenum);
my($i);
my(@errors, @lines, $errorString);
my($shortscopname);
my(@foldresults, $target, $fold, $superfam, $fam);
my($tr, $class, $gray);
my(%foldcounts,%descriptioncounts);

$shortscopname = $query;
if($query =~ /^(.*)\.(.*)$/) { #remove filename extensions
  $shortscopname = $1;
}
$newfilenum = $filenum+1;
if($query =~ /queryfile/) {
  $shortscopname = "query";
}

#find the most common folds, superfams, etc.
%foldcounts = ();
foreach $target (keys(%scores)) {
  ($class, $fold, $superfam, $fam) = split(/\./,$classifications{$target});
  $fold = $class.".".$fold;
  $foldcounts{$fold}++;
}
@foldresults = sort(sort_by_count keys(%foldcounts));

#find the most common descriptions (PDB headers)
%descriptioncounts = ();
foreach $target (keys(%scores)) {
  $descriptioncounts{$description{$target}}++;
}



if($sort == -1 || $sort == 2) {   
  @orderbyscore = keys(%scores);
  @orderbyscore = sort(sort_by_score keys(%scores));
}elsif($sort != -1) {
  if($sort == 0) { #sort by scop fold (most common folds appear first)
    @orderbyscore = sort(sort_by_fold keys(%scores));
  }if($sort == 3) { #sort by statistical significance values (p values)
    @orderbyscore = sort(sort_by_pval keys(%pvalues));
  }elsif($sort == 4) { #sort by number of aligned SSEs
    @orderbyscore = sort(sort_by_SSEs keys(%num_aligned_SSEs));
  }elsif($sort == 5) { #sort by number of aligned residues
    @orderbyscore = sort(sort_by_residues keys(%num_aligned_residues));
  }elsif($sort == 6) { #sort by rmsd
    @orderbyscore = sort(sort_by_rmsd keys(%RMSD));
  }elsif($sort == 7) { #sort by pdb header (most common headers appear first)
    @orderbyscore = sort(sort_by_description keys(%description));
  }
}

#print the results to a text file for download (if desired by user)
open(searchtable,">$outputdir/table$filenum.html") || print STDERR "can't open $outputdir/table$filenum.html for writing: $!\n";
printSearchTableHeader(*searchtable);


#print the search table
print searchtable "<table border=0 cellpadding=2>\n";
print searchtable "<tr>\n";
print searchtable "<td><span class=smallfont>View: </span></td>\n";
print searchtable "<td><a href=\"#folds\">SCOP fold statistics</a></td>\n";
print searchtable "<td><a href=\"$wwwdir/logfile\" class=smallfont>Execution Log (very large file!)</a></td>\n";
print searchtable "<td><a href=\"$wwwdir/search\.sort$filenum\" class=smallfont>Search results (non-html)</a></td>\n";
print searchtable "</tr>\n</table>\n";

#print the column headers
printTableHeaders(*searchtable,$chimesize);

#print a line in the table for each hit
for($i=0;$i<=$#orderbyscore;$i++) {
  printTableLine($shortscopname,$orderbyscore[$i],$i+1, *searchtable,$filenum);
}
print searchtable "</table>\n";

#print fold statistics
print searchtable "<br><a name=\"folds\"></a><table border = 0>\n";
print searchtable "<tr bgcolor=\"#bbbbbb\"><th align=left><a href=\"javascript:launchHelpWindow('selectfold')\" onMouseOver=\"status='Click for help';return true\" onMouseOut=\"status='';return true;\" class=\"helplink\">Select</a></th><th align=left>SCOP Fold</th><th align=left>Count</th></tr>\n";

$gray = 0;
foreach $fold (@foldresults) {
  if(defined($color_fold{$fold})) {
    $checked = "checked";
  }else {
    $checked = "";
  }
  if($gray) {
    $tr = "<tr bgcolor=\"#eeeeee\">";
    $gray = 0;
  }else {
    $tr = "<tr>";
    $gray = 1;
  }
  print searchtable "$tr\n<td align=left><input type=checkbox $checked name=$fold onClick=\"javascript:colorFold()\"></td><td align=left><a href=\"http://scop.berkeley.edu/search.cgi?sccs=$fold\" target=_blank>$foldnames{$fold}</a></td><td align=right><span class=\"searchInfo\">$foldcounts{$fold}</span></td></tr>\n";
}
print searchtable "</table>\n";
print searchtable "</form>\n";
print searchtable "<\/body>\n</html>\n";
close(searchtable);
open(IN,"$outputdir/table$filenum.html");
#open(IN, "table$filenum.html");
@lines = <IN>;
close(IN);
print "Content-type: text/html\n\n";
print @lines;



#This subroutine prints out information about a hit to a line in the searh
#results table
sub printTableLine {
  my($query,$target, $target_rank,$file,$filenum) = @_;
  my($shortpdbname,$shortscopname,$targetscoplink);
  my($score,$pvalue,$description,$rmsd);
  my($url, $yourfile);
  my($row_class,$checked);
  my($classification,$fold);
  
  ($fold) = $classifications{$target} =~ /^([abcd]\.[0-9]+)/;

  if(defined($color_fold{$fold})) {
#    $row_class = "class=\"selectedFold\"";
    $row_class = "bgcolor=yellow";
    $checked = "checked";
  }else {
    $row_class = "";
    $checked = "";
  }

#  print STDERR "printTableLine: rank=$rank\n";exit(0);
  #So far, all search databases consist of SCOP domains
  ($shortpdbname) = $target =~ /^.(....)/;
  ($shortscopname) = $target =~/^(.......)/; #remove any filename extensions

  $targetscoplink = "http://scop.berkeley.edu/search.cgi?sid=$shortscopname";
  print $file "<tr $row_class>\n";

  print $file "<td align=left>$target_rank&nbsp;</td>\n";
  
  print $file "<td align=left>";
  print $file "<a href=$targetscoplink target=$shortscopname>$target</a></td>\n";
  print $file  "<td align=center>";
  print $file  "<a href=$wwwdir/$query\_$target\.out>Text</a>";
  print $file  " ,\n ";
  print $file "<a href=/cgi-bin/FoldMiner2/newpdb.cgi?yourfile=$query\_$shortscopname\.out\&outfile=$query\_$shortscopname\.pdb\&outdir=$outputdir\&wwwdir=$wwwdir\&chime=0>PDB</a>";
  print $file " ,\n ";

  $score = sprintf("%4.2f",$scores{$target});
  $pvalue = sprintf("%1.2g",$pvalues{$target});
  $rmsd = sprintf("%4.2f",$RMSD{$target});
  $description = CGI::escape($description{$target});
  $yourfile = "$query\_$shortscopname\.out";

  print $file  "<a href=\"javascript:viewAlignment('/cgi-bin/FoldMiner/newpdb.cgi?yourfile=$query\_$shortscopname\.out\&outfile=$query\_$shortscopname\.pdb\&outdir=$outputdir\&wwwdir=$wwwdir\&chime=1\&motif=1\&search=1\&filenum=$filenum&score=$score&pval=$pvalue&alignedSSEs=$num_aligned_SSEs{$target}&alignedresidues=$num_aligned_residues{$target}&rmsd=$rmsd&description=$description')\" onMouseOver=\"status='View alignment'\" onMouseOut=\"status=''\">Chime</a>";
  print $file "</td>\n";

  #alignment info 
  printf $file ("<td align=center>%4.2f</td>\n",$scores{$target}); #score
  printf $file ("<td align=center>%1.2g</td>\n",$pvalues{$target}); #p value
  print $file "<td align=center>$num_aligned_SSEs{$target}</td>\n"; #num aligned SSEs
  print $file "<td align=center>$num_aligned_residues{$target}</td>\n"; #number of aligned residues
  print $file "<td align=center>$RMSD{$target}</td>\n"; #RMSD
  print $file "<td align=left><font size=-1>$description{$target}</font></td>\n"; #PDB header
  print $file "</tr>\n";
}

#this subroutine prints the column headers for the search results table
sub printTableHeaders {
  my($file,$chimesize) = @_;
#  print STDERR "chimesize: $chimesize\n";
#  print STDERR "searchiter: $searchiter\n";
#  print STDERR "chime: $chime\n";
  print $file "<form name=\"sortform\" action=\"/cgi-bin/FoldMiner2/formatSearchTable.cgi\">\n<input type=hidden name=sortcol value=2>\n";
  print $file "<input type=hidden name=xval value=$xval>\n";
  print $file "<input type=hidden name=iter value=$searchiter>\n";
  print $file "<input type=hidden name=resultsdir value=\"$wwwdir\">\n";
  print $file "<input type=hidden name=query value=$query>\n";
  print $file "<input type=hidden name=querylink value=\"$querywwwlink\">\n";
  print $file "<input type=hidden name=chime value=$chime>\n";
  print $file "<input type=hidden name=filenum value=$filenum>\n";
  print $file "<input type=hidden name=chimesize value=$chimesize>\n";
  print $file "<input type=hidden name=selectedfolds value=''>\n";
#  print $file "</form>\n";
  print $file "<table width=\"100%\" cellpadding=0 cellspacing=0 border=1>\n";
  print $file " <tr>\n";

  print $file "       <th align=center valign=top>&nbsp;<br><br><br>\n";

  print $file "       <th align=center valign=top><b>Target</b><br><br><a href=\"javascript:sortTable(0)\" class=\"tablesort\">Sort by fold</a></th>\n";
  print $file "       <th align=center valign=top><b><a href=\"javascript:launchHelpWindow('resultsformats')\" onMouseOver=\"status='Click for help';return true\" onMouseOut=\"status='';return true\" class=\"helplink\">View Results</a></b><br><br></th>\n";
  print $file "       <th align=center valign=top><b><a href=\"javascript:launchHelpWindow('alignmentscore')\" onMouseOver=\"status='Click for help';return true\" onMouseOut=\"status='';return true\" class=\"helplink\">Score</a></b><br><br><a href=\"javascript:sortTable(2)\" class=\"tablesort\">Sort</a></th>\n";
  print $file "       <th align=center valign=top><b><a href=\"javascript:launchHelpWindow('significance')\" onMouseOver=\"status='Click for help';return true\" onMouseOut=\"status='';return true\" class=\"helplink\">P value</a></b><br><br><a href=\"javascript:sortTable(3)\" class=\"tablesort\">Sort</a></th>\n";
  print $file "       <th align=center valign=top><b>SSEs</b><br><b>aligned</b><br><a href=\"javascript:sortTable(4)\" class=\"tablesort\">Sort</a></th>\n";
  print $file "       <th align=center valign=top><b>CA Atoms</b><br><b>Aligned</b><br><a href=\"javascript:sortTable(5)\" class=\"tablesort\">Sort</a></th>\n";
  print $file "       <th align=center valign=top><b>RMSD</b><br><br><a href=\"javascript:sortTable(6)\" class=\"tablesort\">Sort</a></th>\n";
  print $file "       <th align=center valign=top><b>PDB HEADER</b><br><br><a href=\"javascript:sortTable(7)\" class=\"tablesort\">Sort</a></th>\n";
  print $file "      </tr>\n";
}


#Prints beginning of the html for the search table page
sub printSearchTableHeader {
  my($file) = @_;
  my($shortscopname);


  if($query =~ /^$resultsdir\/(queryfile)\.pdb$/) {
    $shortscopname = $1;
  }else {
    $shortscopname = $query;
    if($query =~/^(.*)\.(.*)$/) { #remove any filename extensions
      $shortscopname = $1;
    }
  }

  print $file "<html>\n";
  print $file  "<head>\n<title>FoldMiner Search Results Table</title>\n<link rel=stylesheet href=\"/FoldMiner2/lock.css\" type=\"text/css\">\n";
  print $file <<END;
  <script language="Javascript">
    var viewerwin;

    function launchViewer(yourfile,outdir,wwwdir,chime) {
      var URL = "/cgi-bin/FoldMiner/newpdb.cgi?";
      URL += "yourfile="+yourfile;
      URL += "&outdir="+outdir;
      URL += "&resultsdir="+wwwdir;
      URL += "&chime="+chime;
      var mywin=window.open(URL, "alignmentviewer", "directories=no,location=no,menubar=no,toolbar=no,scrollbars=yes,resizable=yes,status=yes");
      mywin.focus();
    }
   
    function colorFold() {
       var str = "";
         for(var i=0;i<document.sortform.elements.length;i++) {
           if(document.sortform.elements[i].type == "checkbox") {
             if(document.sortform.elements[i].checked) {
                str += document.sortform.elements[i].name;
                str += ",";
             }
           }
         }
       str = str.substring(0,str.length-1); //remove last comma
       document.sortform.selectedfolds.value=str;
       document.sortform.submit();
    }

    function sortTable(col) {
       document.sortform.sortcol.value=col;
       colorFold();
//       document.sortform.submit()
    }

    function bringControlToFront() {
       window.opener.top.bringControlToFront();
    }

    function viewAlignment(URL) {
      if(viewerwin == null || viewerwin.closed) {
         viewerwin = window.open('','viewer');
      } 
      viewerwin.location.replace(URL);
      viewerwin.focus();
   }

  function launchHelpWindow(anchor) {
   var URL = "/FoldMiner/help.html#"+anchor;
   var mywin=window.open('','help',"directories=no,location=no,menubar=no,scrollbars=yes,toolbar=no,resizable=yes,status=yes,width=350,height=300");
   mywin.location.replace(URL);
    mywin.focus();
  }
  </script>
</head>
END

  print $file "<body bgcolor=#FFFFFF link=#26256b alink=#877bab vlink=#877bab>\n";
  print $file "<a href=/FoldMiner2/><img align=left src=/FoldMiner2/images/foldminer.gif border=0></a><br clear=\"all\">\n<br>\n";
  print $file "<b>FoldMiner search results for query: <a href=$querywwwlink target=$shortscopname>$shortscopname</a></b><br>\n";
  print $file "<a href=\"javascript:bringControlToFront()\" class=\"smallfont\">(Bring control panel to front)</a><br>\n";

}


#targets with folds with higher counts (folds which appear more often 
#among statistically significant targets) are "less than" other targets.
#Within the same fold, ties are broken by alignment score.
sub sort_by_count {
  my($fold1, $fold2) = ($a, $b);
  if($foldcounts{$b} == $foldcounts{$a}) {
    $foldnames{$fold1} cmp $foldnames{$fold2};
  }else {
    $foldcounts{$b} <=> $foldcounts{$a};
  }
}

#targets with PDB headers (descriptions) that appear more often among
#statistically significant targets are "less than" other targets. Within
# the same description, ties are broken by alignment score.
sub sort_by_description {
  my($domain1, $domain2) = ($a,$b);
  
  if($description{$domain1} ne $description{$domain2}) {
    $descriptioncounts{$description{$domain2}} <=> $descriptioncounts{$description{$domain1}};
  }else { #sort by score within the same description
    $scores{$b} <=> $scores{$a};
  }
  
}

#Sorts targets by the folds that appear most often. Within the same fold,
#targets with higher scores appear first.
sub sort_by_fold {
  my($domain1, $domain2) = ($a, $b);
  my($class1,$fold1, $superfam1,$fam1, $class2, $fold2, $superfam2, $fam2);
  my($foldnum1, $foldnum2);

  ($class1,$foldnum1,$superfam1,$fam1) = split(/\./,$classifications{$a});
  ($class2,$foldnum2,$superfam2,$fam2) = split(/\./,$classifications{$b});
  $fold1 = $class1.".".$foldnum1;
  $fold2 = $class2.".".$foldnum2;
  if($fold1 ne $fold2) {
    if($foldcounts{$fold2} == $foldcounts{$fold1}) {
 #     print "sort by fold: $fold1, $fold2," $fold1 <=> $fold2,"\n";
      $foldnames{$fold1} cmp $foldnames{$fold2};
    }else {
      $foldcounts{$fold2} <=> $foldcounts{$fold1};
    }
  }else { #sort by score within same fold
    $scores{$b} <=> $scores{$a};
  }
  
}


#Sorts folds into a list such that folds in each SCOP class appear together
#in numerical order. The distribution representing all of SCOP appears first,
#and distributions for entire classes appear before distributions for
#individual folds within that class
sub sort_folds {
  my($class1, $class2);
  my($fold1, $fold2);

  #the fold placeholder "all" (representing the overall distribution) is always first
  if($a eq "all") {
    return(-1);
  }elsif($b eq "all") {
    return(1);
  }else {
    ($class1,$fold1) = split(/\./,$a);
    ($class2, $fold2) = split(/\./,$b);
    if($class1 eq $class2) {
      if($fold1 < $fold2) {
	return(-1);
      }elsif($fold2 < $fold1) {
	return(1);
      }else {
	return(0);
      }
    }elsif($class1 lt $class2) {
      return(-1);
    }elsif($class2 lt $class1) {
      return(1);
    }else {
      return(0);
    }
  }

  
}


#The target with the higher score is "less than" the target with the
#lower score
sub sort_by_score {
    $scores{$b} <=> $scores{$a}
}

#The target with the lower p value is "less than" the target with
#the higher p value
sub sort_by_pval {
  $pvalues{$a} <=> $pvalues{$b};
}

#The target with more aligned SSEs is "less than" the other
sub sort_by_SSEs {
  $num_aligned_SSEs{$b} <=> $num_aligned_SSEs{$a};
}

#The target with more aligned residues is "less than" the other
sub sort_by_residues {
  $num_aligned_residues{$b} <=> $num_aligned_residues{$a};
}

#The target with the lower rmsd is "less than" the other
sub sort_by_rmsd {
  $RMSD{$a} <=> $RMSD{$b};
}


sub redirectToErrorHandler {
  my($input,$error,$correctinput,$helptopic,$pid) = @_;
  my($email) = "jessicas\@stanford.edu";
  my($name) = "Jessica Ebert";

  $input=CGI::escape($input);
  $error = CGI::escape($error);
  $correctinput=CGI::escape($correctinput);
  $helptopic=CGI::escape($helptopic);
  $email=CGI::escape($email);
  $name=CGI::escape($name);

  print "Cache-control: no-cache\n";
  print "Content-type: text/html\n\n";
  print <<END;
<html>
<head>
<meta http-equiv="Pragma" content='no-cache'>
<meta http-equiv="refresh" content="0;URL=/cgi-bin/FoldMiner2/reporterror.cgi?error=$error&exampleinput=$correctinput&input=$input&email=$email&name=$name&helptopic=$helptopic&resultsdir=$pid&runtype=search">
<title>FoldMiner: Error</title>
</head>
<body>
An error occurred. If you are not automatically redirected in two seconds, <a href="/cgi-bin/FoldMiner2/reporterror.cgi?error=$error&exampleinput=$correctinput&input=$input&email=$email&name=$name&helptopic=$helptopic&resultsdir=$pid&runtype=search">click here.</a>
</body>
</html>
END

}
